# coding: utf-8

"""
    Canopy.Api

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from canopy.openapi.configuration import Configuration


class ConfigResolvedReferenceData(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'modified_date': 'datetime',
        'user_id': 'object',
        'name': 'str',
        'config_type': 'object',
        'hashes': 'list[ConfigHash]',
        'is_support_session_open': 'bool'
    }

    attribute_map = {
        'modified_date': 'modifiedDate',
        'user_id': 'userId',
        'name': 'name',
        'config_type': 'configType',
        'hashes': 'hashes',
        'is_support_session_open': 'isSupportSessionOpen'
    }

    def __init__(self, modified_date=None, user_id=None, name=None, config_type=None, hashes=None, is_support_session_open=None, local_vars_configuration=None):  # noqa: E501
        """ConfigResolvedReferenceData - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._modified_date = None
        self._user_id = None
        self._name = None
        self._config_type = None
        self._hashes = None
        self._is_support_session_open = None
        self.discriminator = None

        self.modified_date = modified_date
        self.user_id = user_id
        self.name = name
        self.config_type = config_type
        self.hashes = hashes
        self.is_support_session_open = is_support_session_open

    @property
    def modified_date(self):
        """Gets the modified_date of this ConfigResolvedReferenceData.  # noqa: E501


        :return: The modified_date of this ConfigResolvedReferenceData.  # noqa: E501
        :rtype: datetime
        """
        return self._modified_date

    @modified_date.setter
    def modified_date(self, modified_date):
        """Sets the modified_date of this ConfigResolvedReferenceData.


        :param modified_date: The modified_date of this ConfigResolvedReferenceData.  # noqa: E501
        :type modified_date: datetime
        """
        if self.local_vars_configuration.client_side_validation and modified_date is None:  # noqa: E501
            raise ValueError("Invalid value for `modified_date`, must not be `None`")  # noqa: E501

        self._modified_date = modified_date

    @property
    def user_id(self):
        """Gets the user_id of this ConfigResolvedReferenceData.  # noqa: E501


        :return: The user_id of this ConfigResolvedReferenceData.  # noqa: E501
        :rtype: object
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this ConfigResolvedReferenceData.


        :param user_id: The user_id of this ConfigResolvedReferenceData.  # noqa: E501
        :type user_id: object
        """
        if self.local_vars_configuration.client_side_validation and user_id is None:  # noqa: E501
            raise ValueError("Invalid value for `user_id`, must not be `None`")  # noqa: E501

        self._user_id = user_id

    @property
    def name(self):
        """Gets the name of this ConfigResolvedReferenceData.  # noqa: E501


        :return: The name of this ConfigResolvedReferenceData.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ConfigResolvedReferenceData.


        :param name: The name of this ConfigResolvedReferenceData.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def config_type(self):
        """Gets the config_type of this ConfigResolvedReferenceData.  # noqa: E501


        :return: The config_type of this ConfigResolvedReferenceData.  # noqa: E501
        :rtype: object
        """
        return self._config_type

    @config_type.setter
    def config_type(self, config_type):
        """Sets the config_type of this ConfigResolvedReferenceData.


        :param config_type: The config_type of this ConfigResolvedReferenceData.  # noqa: E501
        :type config_type: object
        """
        if self.local_vars_configuration.client_side_validation and config_type is None:  # noqa: E501
            raise ValueError("Invalid value for `config_type`, must not be `None`")  # noqa: E501

        self._config_type = config_type

    @property
    def hashes(self):
        """Gets the hashes of this ConfigResolvedReferenceData.  # noqa: E501


        :return: The hashes of this ConfigResolvedReferenceData.  # noqa: E501
        :rtype: list[ConfigHash]
        """
        return self._hashes

    @hashes.setter
    def hashes(self, hashes):
        """Sets the hashes of this ConfigResolvedReferenceData.


        :param hashes: The hashes of this ConfigResolvedReferenceData.  # noqa: E501
        :type hashes: list[ConfigHash]
        """
        if self.local_vars_configuration.client_side_validation and hashes is None:  # noqa: E501
            raise ValueError("Invalid value for `hashes`, must not be `None`")  # noqa: E501

        self._hashes = hashes

    @property
    def is_support_session_open(self):
        """Gets the is_support_session_open of this ConfigResolvedReferenceData.  # noqa: E501


        :return: The is_support_session_open of this ConfigResolvedReferenceData.  # noqa: E501
        :rtype: bool
        """
        return self._is_support_session_open

    @is_support_session_open.setter
    def is_support_session_open(self, is_support_session_open):
        """Sets the is_support_session_open of this ConfigResolvedReferenceData.


        :param is_support_session_open: The is_support_session_open of this ConfigResolvedReferenceData.  # noqa: E501
        :type is_support_session_open: bool
        """

        self._is_support_session_open = is_support_session_open

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ConfigResolvedReferenceData):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ConfigResolvedReferenceData):
            return True

        return self.to_dict() != other.to_dict()



from typing import Optional, Dict, Sequence, List

import canopy


class StudyTypesCache(object):

    def __init__(
            self,
            client: canopy.openapi.ApiClient,
            authentication: canopy.Authentication,
            sim_version_cache: canopy.TenantSimVersionCache):

        self._client = client
        self._authentication = authentication
        self._sim_version_cache = sim_version_cache

        self._data: Optional[canopy.openapi.GetStudyTypesQueryResult] = None
        self._study_type_definitions_by_study_type: Optional[Dict[str, canopy.openapi.StudyTypeDefinition]] = None
        self._config_type_metadata_by_config_type: Optional[Dict[str, canopy.openapi.ConfigTypeMetadata]] = None

    def reload(self):
        api = canopy.openapi.StudyApi(self._client)
        result: canopy.openapi.GetStudyTypesQueryResult = api.study_get_study_types(tenant_id=self._authentication.tenant_id)
        self._data = result

        study_types: Sequence[canopy.openapi.StudyTypeDefinition] = result.study_types
        self._study_type_definitions_by_study_type = {v.study_type.lower(): v for v in study_types}
        config_types: Sequence[canopy.openapi.ConfigTypeMetadata] = result.config_type_metadata
        self._config_type_metadata_by_config_type = {v.singular_key.lower(): v for v in config_types}

    def get(
            self,
            sim_version: Optional[str] = None) -> canopy.openapi.GetStudyTypesQueryResult:

        if self._data is None:
            self.reload()

        if self._data is None:
            raise RuntimeError('Users for tenant ID not found.')

        if sim_version is None:
            sim_version = self._sim_version_cache.get()

        study_types_for_sim_version: List[canopy.openapi.StudyTypeDefinition] = []
        for study_type in self._data.study_types:
            study_types_for_sim_version.append(
                canopy.get_study_type_definition_for_sim_version(study_type, sim_version))

        return canopy.openapi.GetStudyTypesQueryResult(
            study_types_for_sim_version,
            self._data.sim_types,
            self._data.config_types)

    def get_study_type_definition(
            self,
            study_type: str,
            sim_version: Optional[str] = None) -> canopy.openapi.StudyTypeDefinition:

        if self._data is None:
            self.reload()

        if sim_version is None:
            sim_version = self._sim_version_cache.get()

        study_type_lower = study_type.lower()
        if study_type_lower not in self._study_type_definitions_by_study_type:
            raise canopy.NotFoundError('Study type not found: {}'.format(study_type))

        result = self._study_type_definitions_by_study_type[study_type_lower]
        return canopy.get_study_type_definition_for_sim_version(result, sim_version)

    def get_config_type_metadata(
            self,
            config_type: str) -> canopy.openapi.ConfigTypeMetadata:

        if self._data is None:
            self.reload()

        config_type_lower = config_type.lower()
        if config_type_lower not in self._config_type_metadata_by_config_type:
            raise canopy.NotFoundError('Config type not found: {}'.format(config_type))

        return self._config_type_metadata_by_config_type[config_type_lower]




from typing import Optional

import canopy
import asyncio
import time


async def wait_for_study(
        session: canopy.Session,
        study_id: str,
        tenant_id: Optional[str] = None,
        timeout_seconds: float = 0) -> canopy.StudyResult:

    session.authentication.authenticate()

    if tenant_id is None:
        tenant_id = session.authentication.tenant_id

    study_api = canopy.openapi.StudyApi(session.async_client)

    start_time = time.monotonic()
    while True:
        study_result: canopy.openapi.GetStudyQueryResult = await study_api.study_get_study_metadata(tenant_id, study_id)
        study_document = canopy.get_study_document(session, study_result.study)

        if study_document.completed_job_count == study_document.job_count:
            break

        current_time = time.monotonic()
        elapsed_time = current_time - start_time

        if timeout_seconds > 0 and elapsed_time > timeout_seconds:
            raise TimeoutError('Timed out waiting for study to complete. Process is {}/{} jobs complete.'.format(
                study_document.completed_job_count,
                study_document.job_count))

        await asyncio.sleep(_get_sleep_time_seconds(elapsed_time))

    return canopy.StudyResult(
        session,
        study_result,
        [])


def _get_sleep_time_seconds(current_duration_seconds: float):
    if current_duration_seconds < 30:
        return 3
    elif current_duration_seconds < 150:
        return 6
    elif current_duration_seconds < 300:
        return 15
    return 30



from typing import Optional

import canopy
import getpass
import datetime


class Authentication(object):
    _client: canopy.openapi.ApiClient
    _identity: canopy.openapi.models.GrantTypeHandlerResponse

    def __init__(
            self,
            client: canopy.openapi.ApiClient,
            client_id: Optional[str] = None,
            client_secret: Optional[str] = None,
            username: Optional[str] = None,
            tenant_name: Optional[str] = None,
            password: Optional[str] = None):
        self._client = client
        self._client_id = client_id
        self._client_secret = client_secret
        self._username = username
        self._tenant_name = tenant_name
        self._password = password
        self._identity = None
        self._expires: datetime.datetime = datetime.datetime.min

    def authenticate(self):
        if self._identity is None or self._expires is None:
            self.sign_in()
        elif datetime.datetime.now() >= self._expires:
            self.refresh_access_token()

    def sign_in(self):
        authentication_data = canopy.prompt_for_authentication(
            client_id=self._client_id,
            client_secret=self._client_secret,
            username=self._username,
            tenant_name=self._tenant_name,
            password=self._password)

        self._client_id = authentication_data.client_id
        self._client_secret = authentication_data.client_secret
        self._username = authentication_data.username
        self._tenant_name = authentication_data.tenant_name
        self._password = authentication_data.password

        post_params = {
            'grant_type': 'password',
            'username': self._username,
            'tenant': self._tenant_name,
            'password': self._password,
            'client_id': self._client_id,
            'client_secret': self._client_secret,
        }

        header_params = {
            'Content-Type': 'application/x-www-form-urlencoded'
        }

        response_types_map = {
            200: "GrantTypeHandlerResponse",
        }

        token_result = self._client.call_api(
            '/token',
            'POST',
            post_params=post_params,
            header_params=header_params,
            response_types_map=response_types_map)

        self._identity = token_result[0]
        self.__update_from_identity()

    def refresh_access_token(self):
        if self._identity is None:
            raise RuntimeError('You must call authenticate before refreshing the access token.')

        post_params = {
            'grant_type': 'refresh_token',
            'refresh_token': self._identity.refresh_token,
            'tenant': self._identity.tenant_id,
            'client_id': self._client_id,
            'client_secret': self._client_secret,
        }

        header_params = {
            'Content-Type': 'application/x-www-form-urlencoded'
        }

        response_types_map = {
            200: "GrantTypeHandlerResponse",
        }

        token_result = self._client.call_api(
            '/token',
            'POST',
            post_params=post_params,
            header_params=header_params,
            response_types_map=response_types_map)

        self._identity = token_result[0]
        self.__update_from_identity()

    def __update_from_identity(self):
        self._client.configuration.access_token = self._identity.access_token
        expires_delta = datetime.timedelta(seconds=self._identity.expires_in)
        self._expires = datetime.datetime.now() + expires_delta

    @property
    def tenant_id(self) -> str:
        if self._identity is None:
            raise RuntimeError('You must call authenticate before requesting tenant ID.')

        return self._identity.tenant_id

    @property
    def user_id(self) -> str:
        if self._identity is None:
            raise RuntimeError('You must call authenticate before requesting user ID.')

        return self._identity.user_id



# coding: utf-8

"""
    Canopy.Api

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from canopy.openapi.configuration import Configuration


class WorksheetPutWorksheetRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'properties': 'list[DocumentCustomPropertyData]',
        'outline': 'NewWorksheetDataOutline',
        'notes': 'str'
    }

    attribute_map = {
        'name': 'name',
        'properties': 'properties',
        'outline': 'outline',
        'notes': 'notes'
    }

    def __init__(self, name=None, properties=None, outline=None, notes=None, local_vars_configuration=None):  # noqa: E501
        """WorksheetPutWorksheetRequest - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._name = None
        self._properties = None
        self._outline = None
        self._notes = None
        self.discriminator = None

        self.name = name
        self.properties = properties
        self.outline = outline
        self.notes = notes

    @property
    def name(self):
        """Gets the name of this WorksheetPutWorksheetRequest.  # noqa: E501


        :return: The name of this WorksheetPutWorksheetRequest.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this WorksheetPutWorksheetRequest.


        :param name: The name of this WorksheetPutWorksheetRequest.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def properties(self):
        """Gets the properties of this WorksheetPutWorksheetRequest.  # noqa: E501


        :return: The properties of this WorksheetPutWorksheetRequest.  # noqa: E501
        :rtype: list[DocumentCustomPropertyData]
        """
        return self._properties

    @properties.setter
    def properties(self, properties):
        """Sets the properties of this WorksheetPutWorksheetRequest.


        :param properties: The properties of this WorksheetPutWorksheetRequest.  # noqa: E501
        :type properties: list[DocumentCustomPropertyData]
        """

        self._properties = properties

    @property
    def outline(self):
        """Gets the outline of this WorksheetPutWorksheetRequest.  # noqa: E501


        :return: The outline of this WorksheetPutWorksheetRequest.  # noqa: E501
        :rtype: NewWorksheetDataOutline
        """
        return self._outline

    @outline.setter
    def outline(self, outline):
        """Sets the outline of this WorksheetPutWorksheetRequest.


        :param outline: The outline of this WorksheetPutWorksheetRequest.  # noqa: E501
        :type outline: NewWorksheetDataOutline
        """

        self._outline = outline

    @property
    def notes(self):
        """Gets the notes of this WorksheetPutWorksheetRequest.  # noqa: E501


        :return: The notes of this WorksheetPutWorksheetRequest.  # noqa: E501
        :rtype: str
        """
        return self._notes

    @notes.setter
    def notes(self, notes):
        """Sets the notes of this WorksheetPutWorksheetRequest.


        :param notes: The notes of this WorksheetPutWorksheetRequest.  # noqa: E501
        :type notes: str
        """

        self._notes = notes

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, WorksheetPutWorksheetRequest):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, WorksheetPutWorksheetRequest):
            return True

        return self.to_dict() != other.to_dict()



import unittest
import asyncio

import canopy


class RequestWithRetryTest(unittest.TestCase):

    def test_it_should_return_result_if_no_error(self):
        request = DummyRequest(0)
        result = canopy.run(canopy.request_with_retry(lambda: request.execute('abc'), 'request', True))
        self.assertEqual(result, 'abc')

    def test_it_should_return_result_if_one_error(self):
        request = DummyRequest(1)
        result = canopy.run(canopy.request_with_retry(lambda: request.execute('abc'), 'request', True))
        self.assertEqual(result, 'abc')

    def test_it_should_raise_error_if_two_errors(self):
        request = DummyRequest(2)
        try:
            canopy.run(canopy.request_with_retry(lambda: request.execute('abc'), 'request', True))
            self.fail()
        except asyncio.TimeoutError:
            pass

    def test_it_should_return_none_if_two_errors_and_suppressed(self):
        request = DummyRequest(2)
        result = canopy.run(canopy.request_with_retry(lambda: request.execute('abc'), 'request', False))
        self.assertIsNone(result)


class DummyRequest:

    def __init__(self, error_count: int):
        self.error_count = error_count

    async def execute(self, result: str) -> str:
        self.error_count -= 1
        await asyncio.sleep(0.1)
        if self.error_count >= 0:
            raise asyncio.TimeoutError('Timed out')
        else:
            return result



import asyncio
from asyncio import Future
from typing import List, Optional, Sequence

import canopy
import logging

logger = logging.getLogger(__name__)


async def load_study(
        session: canopy.Session,
        study_id: str,
        sim_type: Optional[str] = None,
        channel_names: Optional[List[str]] = None,
        tenant_id: Optional[str] = None,
        include_study_sim_types: bool = False,
        include_study_full_document: bool = False,
        include_study_inputs: bool = False,
        include_study_scalar_results: bool = False,
        include_job_metadata: bool = False,
        include_job_inputs: bool = False,
        include_job_full_document: bool = False,
        include_job_scalar_results: bool = False,
        include_job_vector_metadata: bool = False,
        sim_version: Optional[str] = None) -> canopy.StudyResult:

    sim_type = canopy.ensure_sim_type_string(sim_type)

    session.authentication.authenticate()

    if tenant_id is None:
        tenant_id = session.authentication.tenant_id

    study_api = canopy.openapi.StudyApi(session.async_client)

    study_result: canopy.openapi.GetStudyQueryResult
    if include_study_inputs or include_study_sim_types or include_study_full_document:
        study_result = await study_api.study_get_study(tenant_id, study_id)
    else:
        study_result = await study_api.study_get_study_metadata(tenant_id, study_id)

    access_information: canopy.openapi.StudyBlobAccessInformation = study_result.access_information
    jobs_access_information: List[canopy.openapi.BlobAccessInformation] = access_information.jobs
    study_document = canopy.get_study_document(session, study_result.study)

    scalar_results_task: Optional[Future[canopy.StudyScalarResults]] = None
    if include_study_scalar_results:
        scalar_results_task = asyncio.ensure_future(canopy.load_study_scalar_results(
            session,
            access_information))

    semaphore = asyncio.Semaphore(session.default_api_concurrency)

    jobs: Optional[Sequence[canopy.StudyJobResult]] = None
    if include_job_metadata \
            or include_job_inputs \
            or include_job_scalar_results \
            or include_job_vector_metadata \
            or (channel_names is not None and len(channel_names) > 0):

        job_tasks: List[Future[canopy.StudyJobResult]] = []
        for index in range(canopy.job_count_to_simulation_count(study_document.job_count)):
            job_task = asyncio.ensure_future(canopy.load_study_job(
                session,
                study_id,
                sim_type,
                channel_names,
                index,
                tenant_id=tenant_id,
                job_access_information=jobs_access_information[index % len(jobs_access_information)],
                semaphore=semaphore,
                include_inputs=include_job_inputs,
                include_full_document=include_job_full_document,
                include_scalar_results=include_job_scalar_results,
                include_vector_metadata=include_job_vector_metadata,
                sim_version=sim_version))
            job_tasks.append(job_task)
        jobs = await asyncio.gather(*job_tasks)

    scalar_results: Optional[canopy.StudyScalarResults] = None
    if scalar_results_task is not None:
        scalar_results = await scalar_results_task

    return canopy.StudyResult(session, study_result, jobs, scalar_results=scalar_results)



# coding: utf-8

"""
    Canopy.Api

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from canopy.openapi.configuration import Configuration


class SimTypeDefinition(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'sim_type': 'str',
        'name': 'str',
        'known_output_files': 'list[str]',
        'state': 'SimTypeState',
        'input_telemetry_channels': 'SimTypeDefinitionInputTelemetryChannels',
        'previous_definitions': 'list[IPreviousDefinitionSimTypeDefinition]'
    }

    attribute_map = {
        'sim_type': 'simType',
        'name': 'name',
        'known_output_files': 'knownOutputFiles',
        'state': 'state',
        'input_telemetry_channels': 'inputTelemetryChannels',
        'previous_definitions': 'previousDefinitions'
    }

    def __init__(self, sim_type=None, name=None, known_output_files=None, state=None, input_telemetry_channels=None, previous_definitions=None, local_vars_configuration=None):  # noqa: E501
        """SimTypeDefinition - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._sim_type = None
        self._name = None
        self._known_output_files = None
        self._state = None
        self._input_telemetry_channels = None
        self._previous_definitions = None
        self.discriminator = None

        self.sim_type = sim_type
        self.name = name
        self.known_output_files = known_output_files
        self.state = state
        self.input_telemetry_channels = input_telemetry_channels
        self.previous_definitions = previous_definitions

    @property
    def sim_type(self):
        """Gets the sim_type of this SimTypeDefinition.  # noqa: E501


        :return: The sim_type of this SimTypeDefinition.  # noqa: E501
        :rtype: str
        """
        return self._sim_type

    @sim_type.setter
    def sim_type(self, sim_type):
        """Sets the sim_type of this SimTypeDefinition.


        :param sim_type: The sim_type of this SimTypeDefinition.  # noqa: E501
        :type sim_type: str
        """
        if self.local_vars_configuration.client_side_validation and sim_type is None:  # noqa: E501
            raise ValueError("Invalid value for `sim_type`, must not be `None`")  # noqa: E501

        self._sim_type = sim_type

    @property
    def name(self):
        """Gets the name of this SimTypeDefinition.  # noqa: E501


        :return: The name of this SimTypeDefinition.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this SimTypeDefinition.


        :param name: The name of this SimTypeDefinition.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def known_output_files(self):
        """Gets the known_output_files of this SimTypeDefinition.  # noqa: E501


        :return: The known_output_files of this SimTypeDefinition.  # noqa: E501
        :rtype: list[str]
        """
        return self._known_output_files

    @known_output_files.setter
    def known_output_files(self, known_output_files):
        """Sets the known_output_files of this SimTypeDefinition.


        :param known_output_files: The known_output_files of this SimTypeDefinition.  # noqa: E501
        :type known_output_files: list[str]
        """
        if self.local_vars_configuration.client_side_validation and known_output_files is None:  # noqa: E501
            raise ValueError("Invalid value for `known_output_files`, must not be `None`")  # noqa: E501

        self._known_output_files = known_output_files

    @property
    def state(self):
        """Gets the state of this SimTypeDefinition.  # noqa: E501


        :return: The state of this SimTypeDefinition.  # noqa: E501
        :rtype: SimTypeState
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this SimTypeDefinition.


        :param state: The state of this SimTypeDefinition.  # noqa: E501
        :type state: SimTypeState
        """

        self._state = state

    @property
    def input_telemetry_channels(self):
        """Gets the input_telemetry_channels of this SimTypeDefinition.  # noqa: E501


        :return: The input_telemetry_channels of this SimTypeDefinition.  # noqa: E501
        :rtype: SimTypeDefinitionInputTelemetryChannels
        """
        return self._input_telemetry_channels

    @input_telemetry_channels.setter
    def input_telemetry_channels(self, input_telemetry_channels):
        """Sets the input_telemetry_channels of this SimTypeDefinition.


        :param input_telemetry_channels: The input_telemetry_channels of this SimTypeDefinition.  # noqa: E501
        :type input_telemetry_channels: SimTypeDefinitionInputTelemetryChannels
        """

        self._input_telemetry_channels = input_telemetry_channels

    @property
    def previous_definitions(self):
        """Gets the previous_definitions of this SimTypeDefinition.  # noqa: E501


        :return: The previous_definitions of this SimTypeDefinition.  # noqa: E501
        :rtype: list[IPreviousDefinitionSimTypeDefinition]
        """
        return self._previous_definitions

    @previous_definitions.setter
    def previous_definitions(self, previous_definitions):
        """Sets the previous_definitions of this SimTypeDefinition.


        :param previous_definitions: The previous_definitions of this SimTypeDefinition.  # noqa: E501
        :type previous_definitions: list[IPreviousDefinitionSimTypeDefinition]
        """
        if self.local_vars_configuration.client_side_validation and previous_definitions is None:  # noqa: E501
            raise ValueError("Invalid value for `previous_definitions`, must not be `None`")  # noqa: E501

        self._previous_definitions = previous_definitions

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SimTypeDefinition):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SimTypeDefinition):
            return True

        return self.to_dict() != other.to_dict()



from typing import Optional

import canopy
import getpass


def prompt_for_authentication(
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
        username: Optional[str] = None,
        tenant_name: Optional[str] = None,
        password: Optional[str] = None) -> canopy.AuthenticationData:

    if client_id is None:
        client_id = input('Client ID:')
    if client_secret is None:
        client_secret = getpass.getpass(prompt='Client Secret:')

    if username is None:
        username = input('Username:')
    if tenant_name is None:
        tenant_name = client_id

    if password is None:
        password = getpass.getpass(prompt='Password:')

    return canopy.AuthenticationData(
        client_id=client_id,
        client_secret=client_secret,
        username=username,
        tenant_name=tenant_name,
        password=password)



from typing import Optional, Dict, List, Union

import canopy


def properties_dict_to_list(properties: Optional[Union[Dict[str, str], object]]) -> List[canopy.openapi.DocumentCustomPropertyData]:
    properties_list: List[canopy.openapi.DocumentCustomPropertyData] = []
    if properties is not None:
        properties = canopy.ensure_dict(properties)
        for name, value in properties.items():
            properties_list.append(canopy.openapi.DocumentCustomPropertyData(
                name=name,
                value=value))

    return properties_list



from typing import Optional, List, Dict

import canopy


def create_list_filter(
        session: canopy.Session,
        is_study: bool,
        name: Optional[str] = None,
        items_per_page: Optional[int] = None,
        owner_username: Optional[str] = None,
        custom_properties: Dict[str, str] = None,
        parent_worksheet_id: Optional[str] = None,
        tenant_id: Optional[str] = None) -> canopy.SerializableValue[canopy.openapi.ListFilter]:

    conditions: List[canopy.openapi.ListFilterCondition] = []

    if name is not None:
        conditions.append(canopy.openapi.ListFilterCondition(
            source='metadata',
            name='name',
            operator='equals',
            value=name
        ))

    if owner_username is not None:
        tenant_users = session.tenant_users.get(tenant_id)
        author = tenant_users.get_by_username(owner_username)
        conditions.append(canopy.openapi.ListFilterCondition(
            source='metadata',
            name='userId',
            operator='equals',
            value=author.user_id
        ))

    if parent_worksheet_id is not None:
        conditions.append(canopy.openapi.ListFilterCondition(
            source='metadata',
            name='parentWorksheetId',
            operator='equals',
            value=parent_worksheet_id
        ))

    if custom_properties is not None:
        for key, value in custom_properties.items():
            conditions.append(canopy.openapi.ListFilterCondition(
                source='customProperty',
                name=key,
                operator='equals',
                value=value
            ))

    query: Optional[canopy.openapi.ListFilterGroup] = None
    if len(conditions) > 0:
        query = canopy.openapi.ListFilterGroup(
                operator='and',
                conditions=conditions)

    return canopy.SerializableValue(
        session,
        canopy.openapi.ListFilter(
            items_per_page=items_per_page,
            order_by_property='creationDate' if is_study else 'modifiedDate',
            order_by_descending=True,
            query=query,
            include_if_has_parent_worksheet=(parent_worksheet_id is not None)))



from typing import Optional, Dict, Any, Union, Sequence, List

import canopy
import logging

logger = logging.getLogger(__name__)


async def create_study(
        session: canopy.Session,
        study_type: str,
        study_name: str,
        inputs: Sequence[Union[str, canopy.LocalConfig, canopy.ConfigResult]],
        properties: Optional[Dict[str, str]] = None,
        notes: Optional[str] = None,
        sim_version: Optional[str] = None,
        is_transient: bool = False) -> str:

    study_type = canopy.ensure_study_type_string(study_type)

    session.authentication.authenticate()
    tenant_id = session.authentication.tenant_id

    study_type_definition = session.study_types.get_study_type_definition(study_type, sim_version)

    provided_inputs = await _resolve_provided_inputs(session, inputs, sim_version)

    sim_config: Dict[str, Any] = {}
    study = {
        'simTypes': study_type_definition.sim_types,
        'simConfig': sim_config
    }

    sources: List[canopy.openapi.NewStudyDataSource] = []
    properties_list: List[canopy.openapi.DocumentCustomPropertyData] = canopy.properties_dict_to_list(properties)

    notes_list: List[str] = []
    if notes is not None and len(notes) > 0:
        notes_list.append(notes)

    input_definitions: Sequence[canopy.openapi.SimulationInput] = study_type_definition.inputs
    for input_definition in input_definitions:
        if input_definition.config_type not in provided_inputs:
            if input_definition.is_required:
                raise RuntimeError('Input {} is required'.format(input_definition.config_type))
        else:
            provided_input = provided_inputs[input_definition.config_type]
            if input_definition.config_type == canopy.Constants.exploration_config_type:
                study['exploration'] = provided_input.raw_data
            else:
                sim_config[input_definition.config_type] = provided_input.raw_data

            sources.append(canopy.openapi.NewStudyDataSource(
                config_type=input_definition.config_type,
                user_id=provided_input.user_id,
                config_id=provided_input.config_id,
                name=provided_input.name,
                is_edited=provided_input.is_edited))

            if provided_input.properties is not None:
                for config_key, value in provided_input.properties.items():
                    key = ''.join([provided_input.config_type, '.', config_key])
                    if key not in properties_list:
                        properties_list.append(canopy.openapi.DocumentCustomPropertyData(
                            name=key,
                            value=value))

            if provided_input.notes is not None and len(provided_input.notes) > 0:
                if len(notes_list) > 0:
                    notes_list.append('\n\n')
                notes_list.append(input_definition.config_type)
                notes_list.append(':\n')
                notes_list.append(provided_input.notes.strip())

    study_api = canopy.openapi.StudyApi(session.async_client)
    study_result: canopy.openapi.PostStudyResult = await study_api.study_post_study(
        tenant_id,
        canopy.openapi.NewStudyData(
            name=study_name,
            study=study,
            is_transient=is_transient,
            study_type=study_type_definition.study_type,
            sources=sources,
            properties=properties_list,
            notes=''.join(notes_list),
            sim_version=sim_version))

    return study_result.study_id


async def _resolve_provided_inputs(session, inputs, sim_version) -> Dict[str, canopy.LocalConfig]:
    provided_inputs: Dict[str, canopy.LocalConfig] = {}
    for i, provided_input in enumerate(inputs):
        if isinstance(provided_input, str):
            provided_input = await canopy.load_config(
                session,
                config_id=provided_input,
                sim_version=sim_version)
            logger.info('Loaded input config {}'.format(provided_input.document.sub_type))

        if isinstance(provided_input, canopy.ConfigResult):
            if provided_input.document.type == canopy.Constants.config_sub_tree_document_type:
                raise RuntimeError('Cannot add a config sub-tree to a study.')

            provided_input = provided_input.to_local_config()

        if not isinstance(provided_input, canopy.LocalConfig):
            raise RuntimeError('Unexpected study input type at position {}'.format(i))

        provided_inputs[provided_input.config_type] = provided_input
    return provided_inputs



from typing import Optional, List, Dict

import canopy
import json


async def find_config(
        session: canopy.Session,
        config_type: str,
        name: Optional[str] = None,
        owner_username: Optional[str] = None,
        custom_properties: Dict[str, str] = None,
        parent_worksheet_id: Optional[str] = None,
        sub_tree_path: Optional[str] = None,
        tenant_id: Optional[str] = None,
        sim_version: Optional[str] = None) -> canopy.ConfigResult:

    session.authentication.authenticate()

    if tenant_id is None:
        tenant_id = session.authentication.tenant_id

    config_api = canopy.openapi.ConfigApi(session.async_client)

    filter_ = canopy.create_list_filter(
        session,
        is_study=False,
        name=name,
        items_per_page=1,
        owner_username=owner_username,
        custom_properties=custom_properties,
        parent_worksheet_id=parent_worksheet_id,
        tenant_id=tenant_id)

    metadata_result: canopy.openapi.GetConfigsQueryResult = await config_api.config_get_configs(
        tenant_id,
        config_type,
        **canopy.defined_kwargs(
            filter=filter_.serialize(),
            sub_tree_path=sub_tree_path))

    documents: List[canopy.openapi.CanopyDocument] = metadata_result.query_results.documents
    if len(documents) == 0:
        raise canopy.NotFoundError('No config found matching the specified criteria.')

    document = documents[0]
    return await canopy.load_config(
        session,
        document.document_id,
        tenant_id=tenant_id,
        sub_tree_path=sub_tree_path,
        sim_version=sim_version)



from typing import Optional, Dict, Any, List

import canopy


async def create_config(
        session: canopy.Session,
        config_type: str,
        name: str,
        config_data: Any,
        properties: Optional[Dict[str, str]] = None,
        notes: Optional[str] = None,
        sim_version: Optional[str] = None,
        sub_tree_path: Optional[str] = None) -> str:

    session.authentication.authenticate()
    tenant_id = session.authentication.tenant_id

    properties_list: List[canopy.openapi.DocumentCustomPropertyData] = canopy.properties_dict_to_list(properties)

    config_api = canopy.openapi.ConfigApi(session.async_client)
    config_id = await config_api.config_post_config(
        tenant_id,
        canopy.openapi.NewConfigData(
            name=name,
            config_type=config_type,
            properties=properties_list,
            config=config_data,
            notes=notes,
            sim_version=sim_version),
        **canopy.defined_kwargs(
            sub_tree_path=sub_tree_path))

    return config_id



from typing import Optional, Callable, Awaitable, Union

import canopy
import aiohttp
import atexit
import asyncio
from aiohttp.client_exceptions import ClientResponseError, ClientConnectionError, ServerTimeoutError, ClientError
import logging
import certifi


logger = logging.getLogger(__name__)


class Session(object):
    _sync_client: canopy.openapi.ApiClient
    _async_client: canopy.openapi_asyncio.ApiClient
    _is_closed: bool = False

    def __init__(
            self,
            authentication_data: Optional[canopy.AuthenticationData] = None,
            client_id: Optional[str] = None,
            client_secret: Optional[str] = None,
            username: Optional[str] = None,
            tenant_name: Optional[str] = None,
            password: Optional[str] = None,
            proxy: Optional[canopy.ProxyConfiguration] = None,
            openapi_configuration: Optional[canopy.openapi.Configuration] = None):

        self._configuration = openapi_configuration if openapi_configuration is not None else canopy.openapi.Configuration()
        if self._configuration.host is None:
            self._configuration.host = 'https://api.canopysimulations.com'
        if proxy is not None:
            self._configuration.proxy = proxy.auth_url
            self._configuration.proxy_headers = proxy.headers

        if self._configuration.ssl_ca_cert is None:
            self._configuration.ssl_ca_cert = certifi.where()
            
        self._sync_client = canopy.openapi.ApiClient(configuration=self._configuration)
        self._async_client = canopy.openapi_asyncio.ApiClient(configuration=self._configuration)

        if authentication_data is not None:
            client_id = authentication_data.client_id
            client_secret = authentication_data.client_secret
            username = authentication_data.username
            tenant_name = authentication_data.tenant_name
            password = authentication_data.password

        self._authentication = canopy.Authentication(
            self._sync_client,
            client_id,
            client_secret,
            username,
            tenant_name,
            password)
        self._units = canopy.Units()
        self._user_settings = canopy.UserSettingsCache(self._sync_client, self._authentication)
        self._tenant_users = canopy.TenantUsersCache(self._sync_client, self._authentication)
        self._tenant_sim_version = canopy.TenantSimVersionCache(self._sync_client, self._authentication)
        self._study_types = canopy.StudyTypesCache(self._sync_client, self._authentication, self._tenant_sim_version)

        atexit.register(self.close)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc, tb):
        canopy.run(self.close())

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.close()

    async def close(self):
        if self._is_closed:
            return

        self._is_closed = True
        await self.async_client.close()
        self.sync_client.close()
        print('closing')
        if hasattr(atexit, 'unregister'):
            atexit.unregister(self.close)

    @property
    def default_api_concurrency(self) -> int:
        return 5

    @property
    def default_blob_storage_concurrency(self) -> int:
        return 5

    @property
    def configuration(self) -> canopy.openapi.Configuration:
        return self._configuration

    @property
    def sync_client(self) -> canopy.openapi.ApiClient:
        return self._sync_client

    @property
    def async_client(self) -> canopy.openapi_asyncio.ApiClient:
        return self._async_client

    @property
    def async_client_session(self) -> aiohttp.ClientSession:
        return self._async_client.rest_client.pool_manager

    @property
    def authentication(self) -> canopy.Authentication:
        return self._authentication

    @property
    def user_settings(self) -> canopy.UserSettingsCache:
        return self._user_settings

    @property
    def tenant_users(self) -> canopy.TenantUsersCache:
        return self._tenant_users

    @property
    def tenant_sim_version(self) -> canopy.TenantSimVersionCache:
        return self._tenant_sim_version

    @property
    def study_types(self) -> canopy.StudyTypesCache:
        return self._study_types

    @property
    def units(self) -> canopy.Units:
        return self._units

    @property
    def async_default_timeout(self) -> aiohttp.ClientSession:
        return self._async_client.rest_client.default_timeout

    async def try_load_text(self, url: str, error_subject: str) -> str:
        return await canopy.request_with_retry(lambda: self._load_text(url), error_subject, False)

    async def try_load_bytes(self, url: str, error_subject: str) -> bytes:
        return await canopy.request_with_retry(lambda: self._load_bytes(url), error_subject, False)

    async def _load_text(self, url: str) -> str:
        async with self._get_async_session(url) as response:
            return await response.text()

    async def _load_bytes(self, url: str) -> bytes:
        async with self._get_async_session(url) as response:
            return await response.read()

    def _get_async_session(self, url: str):
        return self.async_client_session.get(
            url,
            raise_for_status=True,
            timeout=self.async_default_timeout,
            proxy=self.configuration.proxy)



# coding: utf-8

# flake8: noqa

"""
    Canopy.Api

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from canopy.openapi.api.account_settings_api import AccountSettingsApi
from canopy.openapi.api.availability_api import AvailabilityApi
from canopy.openapi.api.config_api import ConfigApi
from canopy.openapi.api.end_to_end_test_inbox_api import EndToEndTestInboxApi
from canopy.openapi.api.list_filter_api import ListFilterApi
from canopy.openapi.api.membership_api import MembershipApi
from canopy.openapi.api.pool_api import PoolApi
from canopy.openapi.api.sim_version_api import SimVersionApi
from canopy.openapi.api.study_api import StudyApi
from canopy.openapi.api.support_session_api import SupportSessionApi
from canopy.openapi.api.tenancy_api import TenancyApi
from canopy.openapi.api.tenant_settings_api import TenantSettingsApi
from canopy.openapi.api.token_api import TokenApi
from canopy.openapi.api.user_settings_api import UserSettingsApi
from canopy.openapi.api.worksheet_api import WorksheetApi

# import ApiClient
from canopy.openapi.api_client import ApiClient
from canopy.openapi.configuration import Configuration
from canopy.openapi.exceptions import OpenApiException
from canopy.openapi.exceptions import ApiTypeError
from canopy.openapi.exceptions import ApiValueError
from canopy.openapi.exceptions import ApiKeyError
from canopy.openapi.exceptions import ApiAttributeError
from canopy.openapi.exceptions import ApiException
# import models into sdk package
from canopy.openapi.models.account_settings_put_request import AccountSettingsPutRequest
from canopy.openapi.models.additional_tests import AdditionalTests
from canopy.openapi.models.admin_tenant_settings import AdminTenantSettings
from canopy.openapi.models.admin_tenant_settings_builder import AdminTenantSettingsBuilder
from canopy.openapi.models.admin_tenant_settings_pool_settings import AdminTenantSettingsPoolSettings
from canopy.openapi.models.allocation_state import AllocationState
from canopy.openapi.models.auto_scale_run import AutoScaleRun
from canopy.openapi.models.auto_scale_run_error import AutoScaleRunError
from canopy.openapi.models.availability_result import AvailabilityResult
from canopy.openapi.models.availability_result_additional_tests import AvailabilityResultAdditionalTests
from canopy.openapi.models.batch_create_configs_result import BatchCreateConfigsResult
from canopy.openapi.models.blob_access_information import BlobAccessInformation
from canopy.openapi.models.canopy_document import CanopyDocument
from canopy.openapi.models.canopy_document_support_session import CanopyDocumentSupportSession
from canopy.openapi.models.channel_import_mapping import ChannelImportMapping
from canopy.openapi.models.channel_settings import ChannelSettings
from canopy.openapi.models.chart_settings import ChartSettings
from canopy.openapi.models.collated_label_definitions import CollatedLabelDefinitions
from canopy.openapi.models.collated_label_definitions_worksheet import CollatedLabelDefinitionsWorksheet
from canopy.openapi.models.collated_worksheet_labels import CollatedWorksheetLabels
from canopy.openapi.models.collated_worksheet_labels_label_definitions import CollatedWorksheetLabelsLabelDefinitions
from canopy.openapi.models.compute_node_result import ComputeNodeResult
from canopy.openapi.models.compute_node_state import ComputeNodeState
from canopy.openapi.models.condition_operator import ConditionOperator
from canopy.openapi.models.config_column_label_definitions import ConfigColumnLabelDefinitions
from canopy.openapi.models.config_decrypt_with_metadata_request import ConfigDecryptWithMetadataRequest
from canopy.openapi.models.config_encrypt_with_metadata_request import ConfigEncryptWithMetadataRequest
from canopy.openapi.models.config_hash import ConfigHash
from canopy.openapi.models.config_owner_data import ConfigOwnerData
from canopy.openapi.models.config_post_config_request import ConfigPostConfigRequest
from canopy.openapi.models.config_post_configs_request import ConfigPostConfigsRequest
from canopy.openapi.models.config_put_config_owner_request import ConfigPutConfigOwnerRequest
from canopy.openapi.models.config_put_config_request import ConfigPutConfigRequest
from canopy.openapi.models.config_reference import ConfigReference
from canopy.openapi.models.config_reference_default import ConfigReferenceDefault
from canopy.openapi.models.config_reference_tenant import ConfigReferenceTenant
from canopy.openapi.models.config_resolved_labels import ConfigResolvedLabels
from canopy.openapi.models.config_resolved_labels_reference import ConfigResolvedLabelsReference
from canopy.openapi.models.config_resolved_reference import ConfigResolvedReference
from canopy.openapi.models.config_resolved_reference_data import ConfigResolvedReferenceData
from canopy.openapi.models.config_type_definition import ConfigTypeDefinition
from canopy.openapi.models.config_type_metadata import ConfigTypeMetadata
from canopy.openapi.models.config_type_state import ConfigTypeState
from canopy.openapi.models.config_upgrade_config_deprecated_request import ConfigUpgradeConfigDeprecatedRequest
from canopy.openapi.models.data_to_decrypt import DataToDecrypt
from canopy.openapi.models.data_to_encrypt import DataToEncrypt
from canopy.openapi.models.default_config_id import DefaultConfigId
from canopy.openapi.models.default_config_reference import DefaultConfigReference
from canopy.openapi.models.default_custom_property_names import DefaultCustomPropertyNames
from canopy.openapi.models.document_custom_property_data import DocumentCustomPropertyData
from canopy.openapi.models.document_custom_property_group import DocumentCustomPropertyGroup
from canopy.openapi.models.document_group_result import DocumentGroupResult
from canopy.openapi.models.document_name_result import DocumentNameResult
from canopy.openapi.models.document_type_custom_property_groups import DocumentTypeCustomPropertyGroups
from canopy.openapi.models.document_user_information import DocumentUserInformation
from canopy.openapi.models.documents_and_continuation_token import DocumentsAndContinuationToken
from canopy.openapi.models.duplicate_configs_data import DuplicateConfigsData
from canopy.openapi.models.duplicate_configs_result import DuplicateConfigsResult
from canopy.openapi.models.email_confirmation_data import EmailConfirmationData
from canopy.openapi.models.file_download_metadata import FileDownloadMetadata
from canopy.openapi.models.get_account_settings_result import GetAccountSettingsResult
from canopy.openapi.models.get_admin_tenant_settings_query_result import GetAdminTenantSettingsQueryResult
from canopy.openapi.models.get_admin_tenant_settings_query_result_settings import GetAdminTenantSettingsQueryResultSettings
from canopy.openapi.models.get_all_support_sessions_query_result import GetAllSupportSessionsQueryResult
from canopy.openapi.models.get_all_support_sessions_query_result_user_information import GetAllSupportSessionsQueryResultUserInformation
from canopy.openapi.models.get_all_tenants_study_statistics_query_result import GetAllTenantsStudyStatisticsQueryResult
from canopy.openapi.models.get_config_names_query_result import GetConfigNamesQueryResult
from canopy.openapi.models.get_config_query_result import GetConfigQueryResult
from canopy.openapi.models.get_config_query_result_config import GetConfigQueryResultConfig
from canopy.openapi.models.get_config_versions_query_result import GetConfigVersionsQueryResult
from canopy.openapi.models.get_configs_query_result import GetConfigsQueryResult
from canopy.openapi.models.get_configs_query_result_query_results import GetConfigsQueryResultQueryResults
from canopy.openapi.models.get_decrypted_data_query_result import GetDecryptedDataQueryResult
from canopy.openapi.models.get_encrypted_data_query_result import GetEncryptedDataQueryResult
from canopy.openapi.models.get_pool_status_query_result import GetPoolStatusQueryResult
from canopy.openapi.models.get_pools_item import GetPoolsItem
from canopy.openapi.models.get_pools_item_interval import GetPoolsItemInterval
from canopy.openapi.models.get_pools_query_result import GetPoolsQueryResult
from canopy.openapi.models.get_sim_version_document_query_result import GetSimVersionDocumentQueryResult
from canopy.openapi.models.get_sim_version_document_query_result_document import GetSimVersionDocumentQueryResultDocument
from canopy.openapi.models.get_sim_version_documents_query_result import GetSimVersionDocumentsQueryResult
from canopy.openapi.models.get_sim_version_downloads_query_result import GetSimVersionDownloadsQueryResult
from canopy.openapi.models.get_studies_query_result import GetStudiesQueryResult
from canopy.openapi.models.get_studies_query_result_query_results import GetStudiesQueryResultQueryResults
from canopy.openapi.models.get_study_download_url_query_result import GetStudyDownloadUrlQueryResult
from canopy.openapi.models.get_study_job_metadata_query_result import GetStudyJobMetadataQueryResult
from canopy.openapi.models.get_study_job_metadata_query_result_access_information import GetStudyJobMetadataQueryResultAccessInformation
from canopy.openapi.models.get_study_job_query_result import GetStudyJobQueryResult
from canopy.openapi.models.get_study_jobs_query_result import GetStudyJobsQueryResult
from canopy.openapi.models.get_study_jobs_query_result_query_results import GetStudyJobsQueryResultQueryResults
from canopy.openapi.models.get_study_query_result import GetStudyQueryResult
from canopy.openapi.models.get_study_query_result_access_information import GetStudyQueryResultAccessInformation
from canopy.openapi.models.get_study_types_query_result import GetStudyTypesQueryResult
from canopy.openapi.models.get_support_session_query_result import GetSupportSessionQueryResult
from canopy.openapi.models.get_tenant_access_information_query_result import GetTenantAccessInformationQueryResult
from canopy.openapi.models.get_tenant_billable_stored_simulation_count_query_result import GetTenantBillableStoredSimulationCountQueryResult
from canopy.openapi.models.get_tenant_channel_import_mappings_query_result import GetTenantChannelImportMappingsQueryResult
from canopy.openapi.models.get_tenant_channel_whitelists_query_result import GetTenantChannelWhitelistsQueryResult
from canopy.openapi.models.get_tenant_channel_whitelists_query_result_settings import GetTenantChannelWhitelistsQueryResultSettings
from canopy.openapi.models.get_tenant_default_custom_property_names_query_result import GetTenantDefaultCustomPropertyNamesQueryResult
from canopy.openapi.models.get_tenant_default_custom_property_names_query_result_settings import GetTenantDefaultCustomPropertyNamesQueryResultSettings
from canopy.openapi.models.get_tenant_query_result import GetTenantQueryResult
from canopy.openapi.models.get_tenant_settings_sim_version_query_result import GetTenantSettingsSimVersionQueryResult
from canopy.openapi.models.get_tenant_study_statistics_query_result import GetTenantStudyStatisticsQueryResult
from canopy.openapi.models.get_tenant_users_query_result import GetTenantUsersQueryResult
from canopy.openapi.models.get_tenant_users_query_result_user_item import GetTenantUsersQueryResultUserItem
from canopy.openapi.models.get_tenant_worksheet_label_definitions_query_result import GetTenantWorksheetLabelDefinitionsQueryResult
from canopy.openapi.models.get_tenant_worksheet_label_definitions_query_result_label_definitions import GetTenantWorksheetLabelDefinitionsQueryResultLabelDefinitions
from canopy.openapi.models.get_tenants_query_result import GetTenantsQueryResult
from canopy.openapi.models.get_tenants_query_result_tenant_item import GetTenantsQueryResultTenantItem
from canopy.openapi.models.get_upvoty_token_query_result import GetUpvotyTokenQueryResult
from canopy.openapi.models.get_user_roles_query_result import GetUserRolesQueryResult
from canopy.openapi.models.get_user_settings_query_result import GetUserSettingsQueryResult
from canopy.openapi.models.get_user_settings_query_result_settings import GetUserSettingsQueryResultSettings
from canopy.openapi.models.get_wiki_document_query_result import GetWikiDocumentQueryResult
from canopy.openapi.models.get_worksheet_query_result import GetWorksheetQueryResult
from canopy.openapi.models.get_worksheet_query_result_worksheet import GetWorksheetQueryResultWorksheet
from canopy.openapi.models.get_zendesk_token_query_result import GetZendeskTokenQueryResult
from canopy.openapi.models.grant_type_handler_response import GrantTypeHandlerResponse
from canopy.openapi.models.group_operator import GroupOperator
from canopy.openapi.models.i_previous_definition_sim_type_definition import IPreviousDefinitionSimTypeDefinition
from canopy.openapi.models.i_previous_definition_sim_type_definition_definition import IPreviousDefinitionSimTypeDefinitionDefinition
from canopy.openapi.models.i_previous_definition_study_type_definition import IPreviousDefinitionStudyTypeDefinition
from canopy.openapi.models.i_previous_definition_study_type_definition_definition import IPreviousDefinitionStudyTypeDefinitionDefinition
from canopy.openapi.models.identified_user_data import IdentifiedUserData
from canopy.openapi.models.label_definition import LabelDefinition
from canopy.openapi.models.label_definitions import LabelDefinitions
from canopy.openapi.models.list_filter import ListFilter
from canopy.openapi.models.list_filter_condition import ListFilterCondition
from canopy.openapi.models.list_filter_group import ListFilterGroup
from canopy.openapi.models.list_filter_query import ListFilterQuery
from canopy.openapi.models.membership_post_email_confirmation_request import MembershipPostEmailConfirmationRequest
from canopy.openapi.models.membership_post_identified_user_request import MembershipPostIdentifiedUserRequest
from canopy.openapi.models.membership_post_password_reset_confirmation_request import MembershipPostPasswordResetConfirmationRequest
from canopy.openapi.models.membership_post_password_reset_request_request import MembershipPostPasswordResetRequestRequest
from canopy.openapi.models.membership_post_registration_request import MembershipPostRegistrationRequest
from canopy.openapi.models.membership_put_user_role_request import MembershipPutUserRoleRequest
from canopy.openapi.models.name_value_pair import NameValuePair
from canopy.openapi.models.new_batch_config_data import NewBatchConfigData
from canopy.openapi.models.new_config_data import NewConfigData
from canopy.openapi.models.new_sim_version_data import NewSimVersionData
from canopy.openapi.models.new_study_data import NewStudyData
from canopy.openapi.models.new_study_data_source import NewStudyDataSource
from canopy.openapi.models.new_tenant_data import NewTenantData
from canopy.openapi.models.new_worksheet_data import NewWorksheetData
from canopy.openapi.models.new_worksheet_data_outline import NewWorksheetDataOutline
from canopy.openapi.models.order_by_property import OrderByProperty
from canopy.openapi.models.password_reset_confirmation_data import PasswordResetConfirmationData
from canopy.openapi.models.password_reset_request_data import PasswordResetRequestData
from canopy.openapi.models.pool_settings import PoolSettings
from canopy.openapi.models.pool_state import PoolState
from canopy.openapi.models.pool_type import PoolType
from canopy.openapi.models.post_study_result import PostStudyResult
from canopy.openapi.models.registration_data import RegistrationData
from canopy.openapi.models.resolved_label import ResolvedLabel
from canopy.openapi.models.resolved_label_numeric_statistics import ResolvedLabelNumericStatistics
from canopy.openapi.models.resolved_statistic_label import ResolvedStatisticLabel
from canopy.openapi.models.sim_type_definition import SimTypeDefinition
from canopy.openapi.models.sim_type_definition_input_telemetry_channels import SimTypeDefinitionInputTelemetryChannels
from canopy.openapi.models.sim_type_input_telemetry_channel import SimTypeInputTelemetryChannel
from canopy.openapi.models.sim_type_input_telemetry_channel_evaluation_definition import SimTypeInputTelemetryChannelEvaluationDefinition
from canopy.openapi.models.sim_type_input_telemetry_channels import SimTypeInputTelemetryChannels
from canopy.openapi.models.sim_type_input_telemetry_evaluated_channel import SimTypeInputTelemetryEvaluatedChannel
from canopy.openapi.models.sim_type_input_telemetry_evaluated_channel_suffix import SimTypeInputTelemetryEvaluatedChannelSuffix
from canopy.openapi.models.sim_type_state import SimTypeState
from canopy.openapi.models.sim_version_document_name_result import SimVersionDocumentNameResult
from canopy.openapi.models.sim_version_post_sim_version_request import SimVersionPostSimVersionRequest
from canopy.openapi.models.simulation_column_label_definitions import SimulationColumnLabelDefinitions
from canopy.openapi.models.simulation_input import SimulationInput
from canopy.openapi.models.simulation_resolved_labels import SimulationResolvedLabels
from canopy.openapi.models.study_blob_access_information import StudyBlobAccessInformation
from canopy.openapi.models.study_document import StudyDocument
from canopy.openapi.models.study_document_study_document_data_source import StudyDocumentStudyDocumentDataSource
from canopy.openapi.models.study_documents_and_continuation_token import StudyDocumentsAndContinuationToken
from canopy.openapi.models.study_input_hash import StudyInputHash
from canopy.openapi.models.study_input_hashes import StudyInputHashes
from canopy.openapi.models.study_job_documents_and_continuation_token import StudyJobDocumentsAndContinuationToken
from canopy.openapi.models.study_post_study_request import StudyPostStudyRequest
from canopy.openapi.models.study_put_study_request import StudyPutStudyRequest
from canopy.openapi.models.study_reference import StudyReference
from canopy.openapi.models.study_resolved_labels import StudyResolvedLabels
from canopy.openapi.models.study_resolved_labels_reference import StudyResolvedLabelsReference
from canopy.openapi.models.study_resolved_reference import StudyResolvedReference
from canopy.openapi.models.study_resolved_reference_data import StudyResolvedReferenceData
from canopy.openapi.models.study_resolved_reference_data_study_document import StudyResolvedReferenceDataStudyDocument
from canopy.openapi.models.study_state import StudyState
from canopy.openapi.models.study_type_definition import StudyTypeDefinition
from canopy.openapi.models.study_type_state import StudyTypeState
from canopy.openapi.models.support_session import SupportSession
from canopy.openapi.models.support_session_data import SupportSessionData
from canopy.openapi.models.support_session_put_support_session_request import SupportSessionPutSupportSessionRequest
from canopy.openapi.models.support_session_response import SupportSessionResponse
from canopy.openapi.models.support_sessions_request_metadata import SupportSessionsRequestMetadata
from canopy.openapi.models.tenancy_post_tenant_request import TenancyPostTenantRequest
from canopy.openapi.models.tenancy_put_tenant_request import TenancyPutTenantRequest
from canopy.openapi.models.tenant_channel_whitelists import TenantChannelWhitelists
from canopy.openapi.models.tenant_config_reference import TenantConfigReference
from canopy.openapi.models.tenant_default_custom_property_names import TenantDefaultCustomPropertyNames
from canopy.openapi.models.tenant_information import TenantInformation
from canopy.openapi.models.tenant_settings_put_admin_tenant_settings_request import TenantSettingsPutAdminTenantSettingsRequest
from canopy.openapi.models.tenant_settings_put_tenant_channel_import_mappings_request import TenantSettingsPutTenantChannelImportMappingsRequest
from canopy.openapi.models.tenant_settings_put_tenant_channel_whitelists_request import TenantSettingsPutTenantChannelWhitelistsRequest
from canopy.openapi.models.tenant_settings_put_tenant_default_custom_property_names_request import TenantSettingsPutTenantDefaultCustomPropertyNamesRequest
from canopy.openapi.models.tenant_settings_put_tenant_settings_sim_version_request import TenantSettingsPutTenantSettingsSimVersionRequest
from canopy.openapi.models.tenant_settings_put_tenant_worksheet_label_definitions_request import TenantSettingsPutTenantWorksheetLabelDefinitionsRequest
from canopy.openapi.models.tenant_statistics import TenantStatistics
from canopy.openapi.models.test_auto_scale_formula_query_result import TestAutoScaleFormulaQueryResult
from canopy.openapi.models.test_auto_scale_formula_query_result_auto_scale_run import TestAutoScaleFormulaQueryResultAutoScaleRun
from canopy.openapi.models.text_document import TextDocument
from canopy.openapi.models.text_document_optional_content import TextDocumentOptionalContent
from canopy.openapi.models.updated_account_settings import UpdatedAccountSettings
from canopy.openapi.models.updated_admin_tenant_settings import UpdatedAdminTenantSettings
from canopy.openapi.models.updated_admin_tenant_settings_settings import UpdatedAdminTenantSettingsSettings
from canopy.openapi.models.updated_channel_import_mappings import UpdatedChannelImportMappings
from canopy.openapi.models.updated_config_data import UpdatedConfigData
from canopy.openapi.models.updated_study_data import UpdatedStudyData
from canopy.openapi.models.updated_tenant_channel_whitelists import UpdatedTenantChannelWhitelists
from canopy.openapi.models.updated_tenant_data import UpdatedTenantData
from canopy.openapi.models.updated_tenant_default_custom_property_names import UpdatedTenantDefaultCustomPropertyNames
from canopy.openapi.models.updated_tenant_settings_sim_version import UpdatedTenantSettingsSimVersion
from canopy.openapi.models.updated_user_settings import UpdatedUserSettings
from canopy.openapi.models.updated_user_settings_settings import UpdatedUserSettingsSettings
from canopy.openapi.models.updated_worksheet_data import UpdatedWorksheetData
from canopy.openapi.models.updated_worksheet_label_definitions import UpdatedWorksheetLabelDefinitions
from canopy.openapi.models.upgrade_config_data import UpgradeConfigData
from canopy.openapi.models.upgrade_config_query_result import UpgradeConfigQueryResult
from canopy.openapi.models.user_information import UserInformation
from canopy.openapi.models.user_role_data import UserRoleData
from canopy.openapi.models.user_settings import UserSettings
from canopy.openapi.models.user_settings_builder import UserSettingsBuilder
from canopy.openapi.models.user_settings_put_user_settings_request import UserSettingsPutUserSettingsRequest
from canopy.openapi.models.versioned_document_metadata import VersionedDocumentMetadata
from canopy.openapi.models.worksheet import Worksheet
from canopy.openapi.models.worksheet_config import WorksheetConfig
from canopy.openapi.models.worksheet_config_reference import WorksheetConfigReference
from canopy.openapi.models.worksheet_outline import WorksheetOutline
from canopy.openapi.models.worksheet_post_duplicate_configs_request import WorksheetPostDuplicateConfigsRequest
from canopy.openapi.models.worksheet_post_worksheet_request import WorksheetPostWorksheetRequest
from canopy.openapi.models.worksheet_put_worksheet_request import WorksheetPutWorksheetRequest
from canopy.openapi.models.worksheet_resolved_labels import WorksheetResolvedLabels
from canopy.openapi.models.worksheet_resolved_references import WorksheetResolvedReferences
from canopy.openapi.models.worksheet_row import WorksheetRow
from canopy.openapi.models.worksheet_row_study import WorksheetRowStudy
from canopy.openapi.models.worksheet_study import WorksheetStudy
from canopy.openapi.models.worksheet_study_reference import WorksheetStudyReference




from typing import Optional


class AuthenticationData(object):
    def __init__(
            self,
            client_id: Optional[str] = None,
            client_secret: Optional[str] = None,
            username: Optional[str] = None,
            tenant_name: Optional[str] = None,
            password: Optional[str] = None):
        self._client_id = client_id
        self._client_secret = client_secret
        self._username = username
        self._tenant_name = tenant_name
        self._password = password

    @property
    def client_id(self) -> Optional[str]:
        return self._client_id

    @property
    def client_secret(self) -> Optional[str]:
        return self._client_secret

    @property
    def username(self) -> Optional[str]:
        return self._username

    @property
    def tenant_name(self) -> Optional[str]:
        return self._tenant_name

    @property
    def password(self) -> Optional[str]:
        return self._password



from typing import Optional


def ensure_sim_type_string(value: Optional[str]) -> Optional[str]:
    if value is None or len(value) == 0 or value[0].isupper():
        return value

    return value[0].upper() + value[1:]



# coding: utf-8

"""
    Canopy.Api

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from canopy.openapi.configuration import Configuration


class StudyResolvedReferenceData(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'creation_date': 'datetime',
        'modified_date': 'datetime',
        'user_id': 'object',
        'name': 'str',
        'study_document': 'StudyResolvedReferenceDataStudyDocument',
        'input_hashes': 'list[StudyInputHashes]',
        'sim_types': 'list[str]',
        'sim_version': 'object',
        'is_support_session_open': 'bool'
    }

    attribute_map = {
        'creation_date': 'creationDate',
        'modified_date': 'modifiedDate',
        'user_id': 'userId',
        'name': 'name',
        'study_document': 'studyDocument',
        'input_hashes': 'inputHashes',
        'sim_types': 'simTypes',
        'sim_version': 'simVersion',
        'is_support_session_open': 'isSupportSessionOpen'
    }

    def __init__(self, creation_date=None, modified_date=None, user_id=None, name=None, study_document=None, input_hashes=None, sim_types=None, sim_version=None, is_support_session_open=None, local_vars_configuration=None):  # noqa: E501
        """StudyResolvedReferenceData - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._creation_date = None
        self._modified_date = None
        self._user_id = None
        self._name = None
        self._study_document = None
        self._input_hashes = None
        self._sim_types = None
        self._sim_version = None
        self._is_support_session_open = None
        self.discriminator = None

        self.creation_date = creation_date
        self.modified_date = modified_date
        self.user_id = user_id
        self.name = name
        self.study_document = study_document
        self.input_hashes = input_hashes
        self.sim_types = sim_types
        self.sim_version = sim_version
        self.is_support_session_open = is_support_session_open

    @property
    def creation_date(self):
        """Gets the creation_date of this StudyResolvedReferenceData.  # noqa: E501


        :return: The creation_date of this StudyResolvedReferenceData.  # noqa: E501
        :rtype: datetime
        """
        return self._creation_date

    @creation_date.setter
    def creation_date(self, creation_date):
        """Sets the creation_date of this StudyResolvedReferenceData.


        :param creation_date: The creation_date of this StudyResolvedReferenceData.  # noqa: E501
        :type creation_date: datetime
        """
        if self.local_vars_configuration.client_side_validation and creation_date is None:  # noqa: E501
            raise ValueError("Invalid value for `creation_date`, must not be `None`")  # noqa: E501

        self._creation_date = creation_date

    @property
    def modified_date(self):
        """Gets the modified_date of this StudyResolvedReferenceData.  # noqa: E501


        :return: The modified_date of this StudyResolvedReferenceData.  # noqa: E501
        :rtype: datetime
        """
        return self._modified_date

    @modified_date.setter
    def modified_date(self, modified_date):
        """Sets the modified_date of this StudyResolvedReferenceData.


        :param modified_date: The modified_date of this StudyResolvedReferenceData.  # noqa: E501
        :type modified_date: datetime
        """
        if self.local_vars_configuration.client_side_validation and modified_date is None:  # noqa: E501
            raise ValueError("Invalid value for `modified_date`, must not be `None`")  # noqa: E501

        self._modified_date = modified_date

    @property
    def user_id(self):
        """Gets the user_id of this StudyResolvedReferenceData.  # noqa: E501


        :return: The user_id of this StudyResolvedReferenceData.  # noqa: E501
        :rtype: object
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this StudyResolvedReferenceData.


        :param user_id: The user_id of this StudyResolvedReferenceData.  # noqa: E501
        :type user_id: object
        """
        if self.local_vars_configuration.client_side_validation and user_id is None:  # noqa: E501
            raise ValueError("Invalid value for `user_id`, must not be `None`")  # noqa: E501

        self._user_id = user_id

    @property
    def name(self):
        """Gets the name of this StudyResolvedReferenceData.  # noqa: E501


        :return: The name of this StudyResolvedReferenceData.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this StudyResolvedReferenceData.


        :param name: The name of this StudyResolvedReferenceData.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def study_document(self):
        """Gets the study_document of this StudyResolvedReferenceData.  # noqa: E501


        :return: The study_document of this StudyResolvedReferenceData.  # noqa: E501
        :rtype: StudyResolvedReferenceDataStudyDocument
        """
        return self._study_document

    @study_document.setter
    def study_document(self, study_document):
        """Sets the study_document of this StudyResolvedReferenceData.


        :param study_document: The study_document of this StudyResolvedReferenceData.  # noqa: E501
        :type study_document: StudyResolvedReferenceDataStudyDocument
        """
        if self.local_vars_configuration.client_side_validation and study_document is None:  # noqa: E501
            raise ValueError("Invalid value for `study_document`, must not be `None`")  # noqa: E501

        self._study_document = study_document

    @property
    def input_hashes(self):
        """Gets the input_hashes of this StudyResolvedReferenceData.  # noqa: E501


        :return: The input_hashes of this StudyResolvedReferenceData.  # noqa: E501
        :rtype: list[StudyInputHashes]
        """
        return self._input_hashes

    @input_hashes.setter
    def input_hashes(self, input_hashes):
        """Sets the input_hashes of this StudyResolvedReferenceData.


        :param input_hashes: The input_hashes of this StudyResolvedReferenceData.  # noqa: E501
        :type input_hashes: list[StudyInputHashes]
        """
        if self.local_vars_configuration.client_side_validation and input_hashes is None:  # noqa: E501
            raise ValueError("Invalid value for `input_hashes`, must not be `None`")  # noqa: E501

        self._input_hashes = input_hashes

    @property
    def sim_types(self):
        """Gets the sim_types of this StudyResolvedReferenceData.  # noqa: E501


        :return: The sim_types of this StudyResolvedReferenceData.  # noqa: E501
        :rtype: list[str]
        """
        return self._sim_types

    @sim_types.setter
    def sim_types(self, sim_types):
        """Sets the sim_types of this StudyResolvedReferenceData.


        :param sim_types: The sim_types of this StudyResolvedReferenceData.  # noqa: E501
        :type sim_types: list[str]
        """
        if self.local_vars_configuration.client_side_validation and sim_types is None:  # noqa: E501
            raise ValueError("Invalid value for `sim_types`, must not be `None`")  # noqa: E501

        self._sim_types = sim_types

    @property
    def sim_version(self):
        """Gets the sim_version of this StudyResolvedReferenceData.  # noqa: E501


        :return: The sim_version of this StudyResolvedReferenceData.  # noqa: E501
        :rtype: object
        """
        return self._sim_version

    @sim_version.setter
    def sim_version(self, sim_version):
        """Sets the sim_version of this StudyResolvedReferenceData.


        :param sim_version: The sim_version of this StudyResolvedReferenceData.  # noqa: E501
        :type sim_version: object
        """
        if self.local_vars_configuration.client_side_validation and sim_version is None:  # noqa: E501
            raise ValueError("Invalid value for `sim_version`, must not be `None`")  # noqa: E501

        self._sim_version = sim_version

    @property
    def is_support_session_open(self):
        """Gets the is_support_session_open of this StudyResolvedReferenceData.  # noqa: E501


        :return: The is_support_session_open of this StudyResolvedReferenceData.  # noqa: E501
        :rtype: bool
        """
        return self._is_support_session_open

    @is_support_session_open.setter
    def is_support_session_open(self, is_support_session_open):
        """Sets the is_support_session_open of this StudyResolvedReferenceData.


        :param is_support_session_open: The is_support_session_open of this StudyResolvedReferenceData.  # noqa: E501
        :type is_support_session_open: bool
        """

        self._is_support_session_open = is_support_session_open

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, StudyResolvedReferenceData):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, StudyResolvedReferenceData):
            return True

        return self.to_dict() != other.to_dict()



from typing import Optional, Sequence

from aiohttp.client_exceptions import ClientResponseError
import canopy
import pandas as pd
from io import StringIO
import logging

logger = logging.getLogger(__name__)

units_column_names = ['units']


async def try_load_csv_from_url(
        session: canopy.Session,
        url: str,
        file_name: str,
        index_column_name: Optional[str] = None,
        empty_string_valid_column_names: Optional[Sequence[str]] = None) -> Optional[pd.DataFrame]:

    text: Optional[str] = await session.try_load_text(url, file_name)
    if text is None:
        return None

    data_frame = pd.read_csv(StringIO(text), index_col=False)
    canopy.process_data_frame(data_frame, index_column_name, empty_string_valid_column_names)
    return data_frame



from asyncio import Future
from typing import List, Optional

import asyncio
import pytest
import logging

import canopy
import integration_tests

logger = logging.getLogger(__name__)

default_car_name = 'Canopy F1 Car 2019'
default_weather_name = '25 deg, dry'
default_exploration_name = 'Automated Test Monte Carlo'
test_car_name = 'Python Automated Test Car'
test_weather_name = 'Python Automated Test Weather'
test_study_name = 'Python Automated Test Study'
test_study_name_2 = 'Python Automated Test Study 2'
test_study_property_key = 'python-automated-test'
test_study_property_value = 'true'
test_custom_properties = {
    test_study_property_key: test_study_property_value,
}
test_passed_though_custom_properties = {
    'weather.' + test_study_property_key: test_study_property_value,
    'car.' + test_study_property_key: test_study_property_value,
}
exploration_notes = 'Exploration notes.'


class State(object):
    configs_to_remove: List[str] = []
    studies_to_remove: List[str] = []
    default_car: Optional[canopy.LocalConfig] = None
    default_weather: Optional[canopy.LocalConfig] = None
    study_id: Optional[str] = None
    study_id_2: Optional[str] = None


@pytest.mark.asyncio
class TestStudy:

    @pytest.fixture(autouse=True, scope='class')
    def state(self) -> State:
        return State()

    async def wait_for_and_verify_study(
            self,
            state: State,
            environment: integration_tests.Environment,
            study_id: str,
            expected_name: str) -> canopy.StudyResult:
        assert study_id is not None
        state.studies_to_remove.append(study_id)

        wait_result = await canopy.wait_for_study(
            environment.session,
            study_id,
            timeout_seconds=120)

        assert expected_name == wait_result.document.name
        assert wait_result.data.job_count > 0
        assert wait_result.data.job_count == wait_result.data.completed_job_count
        return wait_result

    async def test_0100_it_should_create_a_study_from_local_configs(self, state: State):
        async with integration_tests.Environment() as environment:
            state.default_car = await canopy.load_default_config(
                environment.session,
                'car',
                default_car_name)
            state.default_weather = await canopy.load_default_config(
                environment.session,
                'weather',
                default_weather_name)

            state.study_id = await canopy.create_study(
                environment.session,
                'ApexSim',  # Intentionally wrong casing.
                test_study_name,
                [
                    state.default_car,
                    state.default_weather,
                ],
                test_custom_properties)
            logger.info('Study ID is {}'.format(state.study_id))

            study = await self.wait_for_and_verify_study(state, environment, state.study_id, test_study_name)

            properties = canopy.ensure_dict(study.document.properties)
            assert test_study_property_value == properties[test_study_property_key]
            assert study.simulation_count == 1
            assert study.succeeded_simulation_count == 1
            assert study.data.job_count == 1

    async def test_0200_it_should_create_a_study_from_references_and_loaded_configs(self, state: State):
        async with integration_tests.Environment() as environment:
            car_config_id = await canopy.create_config(
                environment.session,
                'car',
                test_car_name,
                state.default_car.raw_data,
                test_custom_properties)
            weather_config_id = await canopy.create_config(
                environment.session,
                'weather',
                test_weather_name,
                state.default_weather.raw_data,
                test_custom_properties)
            default_exploration = await canopy.load_default_config(
                environment.session,
                'exploration',
                default_exploration_name)

            default_exploration.data.design.numberOfPoints = 2
            default_exploration.properties = {
                'type': 'a'
            }
            default_exploration.notes = exploration_notes

            weather_config = await canopy.load_config(
                environment.session, weather_config_id)

            state.study_id_2 = await canopy.create_study(
                environment.session,
                'apexSim',
                test_study_name_2,
                [
                    car_config_id,
                    weather_config,
                    default_exploration,
                ])
            logger.info('Study ID 2 is {}'.format(state.study_id_2))

            study = await self.wait_for_and_verify_study(state, environment, state.study_id_2, test_study_name_2)

            properties = canopy.ensure_dict(study.document.properties)
            assert test_study_property_value == properties['car.' + test_study_property_key]
            assert test_study_property_value == properties['weather.' + test_study_property_key]
            assert 'a' == properties['exploration.type']
            assert study.simulation_count == 2
            assert study.succeeded_simulation_count == 2
            assert study.data.job_count == 3

    async def test_0300_it_should_find_a_study_by_name(self, state: State):
        async with integration_tests.Environment() as environment:
            study = await canopy.find_study(
                environment.session,
                owner_username=environment.data.authentication.username,
                name=test_study_name)

            assert study.document.document_id == state.study_id

    async def test_0400_it_should_find_a_study_by_custom_properties(self, state: State):
        async with integration_tests.Environment() as environment:
            study = await canopy.find_study(
                environment.session,
                owner_username=environment.data.authentication.username,
                custom_properties=test_passed_though_custom_properties)

            assert study.document.document_id == state.study_id_2
            assert study.jobs is None

    async def test_0500_it_should_find_a_study_by_multiple_criteria(self, state: State):
        async with integration_tests.Environment() as environment:
            study = await canopy.find_study(
                environment.session,
                owner_username=environment.data.authentication.username,
                name=test_study_name_2,
                custom_properties=test_passed_though_custom_properties)

            assert study.document.document_id == state.study_id_2
            assert study.jobs is None

    async def test_0600_it_should_load_a_study_by_id(self, state: State):
        async with integration_tests.Environment() as environment:
            study = await canopy.load_study(
                environment.session,
                state.study_id)

            assert study.document.document_id == state.study_id
            assert study.inputs is None
            assert study.sim_types is None
            assert study.scalar_results is None

            assert study.jobs is None

    async def test_0610_it_should_load_a_study_by_id_with_data_when_no_study_scalar_results(self, state: State):
        async with integration_tests.Environment() as environment:
            study = await canopy.load_study(
                environment.session,
                state.study_id,
                include_study_full_document=True,
                include_study_scalar_results=True)

            assert study.document.document_id == state.study_id
            assert study.inputs is not None
            assert study.sim_types is not None
            assert study.scalar_results is not None
            assert study.scalar_results.results is None
            assert study.scalar_results.results_metadata is None
            assert study.scalar_results.inputs is None
            assert study.scalar_results.inputs_metadata is None

            assert study.scalar_as('hRideF200:ApexSim', 'mm') is None
            assert study.scalar_as('car.chassis.hRideFSetup+', 'mm') is None

            assert study.jobs is None

    async def test_0611_it_should_load_a_study_by_id_with_data_when_study_scalar_results_exist(self, state: State):
        async with integration_tests.Environment() as environment:
            study = await canopy.load_study(
                environment.session,
                state.study_id_2,
                include_study_full_document=True,
                include_study_scalar_results=True)

            assert study.document.document_id == state.study_id_2
            assert study.inputs is not None
            assert study.sim_types is not None
            assert study.scalar_results is not None
            assert study.scalar_results.results is not None
            assert study.scalar_results.results_metadata is not None
            assert study.scalar_results.inputs is not None
            assert study.scalar_results.inputs_metadata is not None

            assert len(study.scalar_as('hRideF200:ApexSim', 'mm')) == 2
            assert len(study.scalar_as('car.chassis.hRideFSetup+', 'mm')) == 2
            assert study.scalar_results.units['hRideF200:ApexSim'] == 'm'
            assert study.scalar_results.units['car.chassis.hRideFSetup+'] == 'm'

            # Notes are only loaded with the full study document.
            assert exploration_notes in study.document.notes

            assert study.jobs is None

    async def test_0620_it_should_load_a_study_by_id_with_job_metadata(self, state: State):
        async with integration_tests.Environment() as environment:
            study = await canopy.load_study(
                environment.session,
                state.study_id,
                include_job_metadata=True)

            assert study.document.document_id == state.study_id
            assert 1 == len(study.jobs)

            job = study.jobs[0]
            assert job.result is not None
            assert job.document is not None
            assert job.inputs is None
            assert 0 == len(job.scalar_data)
            assert 0 == len(job.vector_data.columns)
            assert 0 == len(job.vector_data)
            assert job.vector_metadata is None

    async def test_0700_it_should_load_a_study_by_id_with_data(self, state: State):
        async with integration_tests.Environment() as environment:
            study = await canopy.load_study(
                environment.session,
                state.study_id,
                sim_type='apexSim',  # Intentionally wrong casing.
                channel_names=[
                    'vCar',
                    'tLap',  # This channel won't exist.
                    'hRideF',
                ],
                include_job_inputs=True,
                include_job_scalar_results=True)

            assert study.document.document_id == state.study_id
            assert 1 == len(study.jobs)

            job = study.jobs[0]
            assert job.result is not None
            assert job.document is not None
            assert job.inputs is not None
            assert len(job.scalar_data) > 0
            assert len(job.vector_data.columns) == 2
            assert len(job.vector_data) > 10
            assert len(job.vector_metadata) > 0

            assert len(job.vector_as('vCar', 'kph')) > 0
            assert job.vector_units['vCar'] == 'm/s'

            assert job.scalar_as('hRideF200', 'mm') > 0
            assert job.scalar_units['hRideF200'] == 'm'

    async def test_0800_it_should_load_a_study_by_id_with_job_vector_metadata_only(self, state: State):
        async with integration_tests.Environment() as environment:
            study = await canopy.load_study(
                environment.session,
                state.study_id,
                sim_type='ApexSim',
                include_job_vector_metadata=True)

            assert study.document.document_id == state.study_id
            assert 1 == len(study.jobs)

            job = study.jobs[0]
            assert 0 == len(job.scalar_data)
            assert 0 == len(job.vector_data.columns)
            assert 0 == len(job.vector_data)
            assert len(job.vector_metadata) > 0

    async def test_9000_it_should_remove_configs(self, state: State):
        async with integration_tests.Environment() as environment:
            tasks: List[Future] = []
            for config_id in state.configs_to_remove:
                tasks.append(asyncio.ensure_future(canopy.delete_config(
                    environment.session,
                    config_id)))
            await asyncio.gather(*tasks)

    async def test_9100_it_should_remove_studies(self, state: State):
        async with integration_tests.Environment() as environment:
            tasks: List[Future] = []
            for study_id in state.studies_to_remove:
                tasks.append(asyncio.ensure_future(canopy.delete_study(
                    environment.session,
                    study_id)))
            await asyncio.gather(*tasks)



# coding: utf-8

"""
    Canopy API

    Combination of generated code from Open API definitions, and useful helper functions.
"""

# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

from setuptools import setup, find_packages  # noqa: H301

with open('README.md') as f:
    long_description = f.read()

NAME = "canopy"
VERSION = "8.32"

REQUIRES = [
    "numpy",
    "certifi>=2017.4.17",
    "six>=1.10",
    "python-dateutil>=2.1",
    "urllib3>=1.23",
    "pandas>=0.25.1",
    "aiohttp",
    "munch"
]

setup(
    name=NAME,
    packages=find_packages(),
    version=VERSION,
    license='MIT',
    description="Python Client for the Canopy Simulation API",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='James Thurley',
    author_email="james.thurley@canopysimulations.com",
    url="https://github.com/canopysimulations/canopy-python/",
    keywords=["Canopy API", "Canopy Simulations", "Canopy Client"],
    install_requires=REQUIRES,
    #include_package_data=True,
    classifiers=[
        'Development Status :: 3 - Alpha',      # "3 - Alpha", "4 - Beta" or "5 - Production/Stable"
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)



from typing import Optional, Any, List

import canopy


async def update_config(
        session: canopy.Session,
        config: canopy.openapi.CanopyDocument,
        config_data: Optional[Any]):

    config_api = canopy.openapi.ConfigApi(session.async_client)

    properties_list = canopy.properties_dict_to_list(config.properties)
    updated_config_data = canopy.openapi.UpdatedConfigData(
        config.name,
        config.sub_type,
        properties_list,
        config_data if config_data is not None else config.data,
        config.notes,
        config.sim_version)

    await config_api.config_put_config(
        config.tenant_id,
        config.document_id,
        updated_config_data,
        **canopy.defined_kwargs(
            sub_tree_path=config.data['path'] if config.type == canopy.Constants.config_sub_tree_document_type else None))



# coding: utf-8

"""
    Canopy.Api

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from canopy.openapi.configuration import Configuration


class StudyTypeDefinition(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'study_type': 'str',
        'name': 'str',
        'sim_types': 'list[str]',
        'inputs': 'list[SimulationInput]',
        'pool_type': 'PoolType',
        'build_pool_type': 'PoolType',
        'state': 'StudyTypeState',
        'valid_for_transient': 'bool',
        'valid_for_inline': 'bool',
        'previous_definitions': 'list[IPreviousDefinitionStudyTypeDefinition]',
        'implicit_sim_types': 'list[str]'
    }

    attribute_map = {
        'study_type': 'studyType',
        'name': 'name',
        'sim_types': 'simTypes',
        'inputs': 'inputs',
        'pool_type': 'poolType',
        'build_pool_type': 'buildPoolType',
        'state': 'state',
        'valid_for_transient': 'validForTransient',
        'valid_for_inline': 'validForInline',
        'previous_definitions': 'previousDefinitions',
        'implicit_sim_types': 'implicitSimTypes'
    }

    def __init__(self, study_type=None, name=None, sim_types=None, inputs=None, pool_type=None, build_pool_type=None, state=None, valid_for_transient=None, valid_for_inline=None, previous_definitions=None, implicit_sim_types=None, local_vars_configuration=None):  # noqa: E501
        """StudyTypeDefinition - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._study_type = None
        self._name = None
        self._sim_types = None
        self._inputs = None
        self._pool_type = None
        self._build_pool_type = None
        self._state = None
        self._valid_for_transient = None
        self._valid_for_inline = None
        self._previous_definitions = None
        self._implicit_sim_types = None
        self.discriminator = None

        self.study_type = study_type
        self.name = name
        self.sim_types = sim_types
        self.inputs = inputs
        self.pool_type = pool_type
        self.build_pool_type = build_pool_type
        self.state = state
        self.valid_for_transient = valid_for_transient
        self.valid_for_inline = valid_for_inline
        self.previous_definitions = previous_definitions
        self.implicit_sim_types = implicit_sim_types

    @property
    def study_type(self):
        """Gets the study_type of this StudyTypeDefinition.  # noqa: E501


        :return: The study_type of this StudyTypeDefinition.  # noqa: E501
        :rtype: str
        """
        return self._study_type

    @study_type.setter
    def study_type(self, study_type):
        """Sets the study_type of this StudyTypeDefinition.


        :param study_type: The study_type of this StudyTypeDefinition.  # noqa: E501
        :type study_type: str
        """
        if self.local_vars_configuration.client_side_validation and study_type is None:  # noqa: E501
            raise ValueError("Invalid value for `study_type`, must not be `None`")  # noqa: E501

        self._study_type = study_type

    @property
    def name(self):
        """Gets the name of this StudyTypeDefinition.  # noqa: E501


        :return: The name of this StudyTypeDefinition.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this StudyTypeDefinition.


        :param name: The name of this StudyTypeDefinition.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def sim_types(self):
        """Gets the sim_types of this StudyTypeDefinition.  # noqa: E501


        :return: The sim_types of this StudyTypeDefinition.  # noqa: E501
        :rtype: list[str]
        """
        return self._sim_types

    @sim_types.setter
    def sim_types(self, sim_types):
        """Sets the sim_types of this StudyTypeDefinition.


        :param sim_types: The sim_types of this StudyTypeDefinition.  # noqa: E501
        :type sim_types: list[str]
        """
        if self.local_vars_configuration.client_side_validation and sim_types is None:  # noqa: E501
            raise ValueError("Invalid value for `sim_types`, must not be `None`")  # noqa: E501

        self._sim_types = sim_types

    @property
    def inputs(self):
        """Gets the inputs of this StudyTypeDefinition.  # noqa: E501


        :return: The inputs of this StudyTypeDefinition.  # noqa: E501
        :rtype: list[SimulationInput]
        """
        return self._inputs

    @inputs.setter
    def inputs(self, inputs):
        """Sets the inputs of this StudyTypeDefinition.


        :param inputs: The inputs of this StudyTypeDefinition.  # noqa: E501
        :type inputs: list[SimulationInput]
        """
        if self.local_vars_configuration.client_side_validation and inputs is None:  # noqa: E501
            raise ValueError("Invalid value for `inputs`, must not be `None`")  # noqa: E501

        self._inputs = inputs

    @property
    def pool_type(self):
        """Gets the pool_type of this StudyTypeDefinition.  # noqa: E501


        :return: The pool_type of this StudyTypeDefinition.  # noqa: E501
        :rtype: PoolType
        """
        return self._pool_type

    @pool_type.setter
    def pool_type(self, pool_type):
        """Sets the pool_type of this StudyTypeDefinition.


        :param pool_type: The pool_type of this StudyTypeDefinition.  # noqa: E501
        :type pool_type: PoolType
        """

        self._pool_type = pool_type

    @property
    def build_pool_type(self):
        """Gets the build_pool_type of this StudyTypeDefinition.  # noqa: E501


        :return: The build_pool_type of this StudyTypeDefinition.  # noqa: E501
        :rtype: PoolType
        """
        return self._build_pool_type

    @build_pool_type.setter
    def build_pool_type(self, build_pool_type):
        """Sets the build_pool_type of this StudyTypeDefinition.


        :param build_pool_type: The build_pool_type of this StudyTypeDefinition.  # noqa: E501
        :type build_pool_type: PoolType
        """

        self._build_pool_type = build_pool_type

    @property
    def state(self):
        """Gets the state of this StudyTypeDefinition.  # noqa: E501


        :return: The state of this StudyTypeDefinition.  # noqa: E501
        :rtype: StudyTypeState
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this StudyTypeDefinition.


        :param state: The state of this StudyTypeDefinition.  # noqa: E501
        :type state: StudyTypeState
        """

        self._state = state

    @property
    def valid_for_transient(self):
        """Gets the valid_for_transient of this StudyTypeDefinition.  # noqa: E501


        :return: The valid_for_transient of this StudyTypeDefinition.  # noqa: E501
        :rtype: bool
        """
        return self._valid_for_transient

    @valid_for_transient.setter
    def valid_for_transient(self, valid_for_transient):
        """Sets the valid_for_transient of this StudyTypeDefinition.


        :param valid_for_transient: The valid_for_transient of this StudyTypeDefinition.  # noqa: E501
        :type valid_for_transient: bool
        """
        if self.local_vars_configuration.client_side_validation and valid_for_transient is None:  # noqa: E501
            raise ValueError("Invalid value for `valid_for_transient`, must not be `None`")  # noqa: E501

        self._valid_for_transient = valid_for_transient

    @property
    def valid_for_inline(self):
        """Gets the valid_for_inline of this StudyTypeDefinition.  # noqa: E501


        :return: The valid_for_inline of this StudyTypeDefinition.  # noqa: E501
        :rtype: bool
        """
        return self._valid_for_inline

    @valid_for_inline.setter
    def valid_for_inline(self, valid_for_inline):
        """Sets the valid_for_inline of this StudyTypeDefinition.


        :param valid_for_inline: The valid_for_inline of this StudyTypeDefinition.  # noqa: E501
        :type valid_for_inline: bool
        """
        if self.local_vars_configuration.client_side_validation and valid_for_inline is None:  # noqa: E501
            raise ValueError("Invalid value for `valid_for_inline`, must not be `None`")  # noqa: E501

        self._valid_for_inline = valid_for_inline

    @property
    def previous_definitions(self):
        """Gets the previous_definitions of this StudyTypeDefinition.  # noqa: E501


        :return: The previous_definitions of this StudyTypeDefinition.  # noqa: E501
        :rtype: list[IPreviousDefinitionStudyTypeDefinition]
        """
        return self._previous_definitions

    @previous_definitions.setter
    def previous_definitions(self, previous_definitions):
        """Sets the previous_definitions of this StudyTypeDefinition.


        :param previous_definitions: The previous_definitions of this StudyTypeDefinition.  # noqa: E501
        :type previous_definitions: list[IPreviousDefinitionStudyTypeDefinition]
        """
        if self.local_vars_configuration.client_side_validation and previous_definitions is None:  # noqa: E501
            raise ValueError("Invalid value for `previous_definitions`, must not be `None`")  # noqa: E501

        self._previous_definitions = previous_definitions

    @property
    def implicit_sim_types(self):
        """Gets the implicit_sim_types of this StudyTypeDefinition.  # noqa: E501


        :return: The implicit_sim_types of this StudyTypeDefinition.  # noqa: E501
        :rtype: list[str]
        """
        return self._implicit_sim_types

    @implicit_sim_types.setter
    def implicit_sim_types(self, implicit_sim_types):
        """Sets the implicit_sim_types of this StudyTypeDefinition.


        :param implicit_sim_types: The implicit_sim_types of this StudyTypeDefinition.  # noqa: E501
        :type implicit_sim_types: list[str]
        """

        self._implicit_sim_types = implicit_sim_types

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, StudyTypeDefinition):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, StudyTypeDefinition):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Canopy.Api

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from canopy.openapi.configuration import Configuration


class GetConfigQueryResultConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'document_id': 'object',
        'tenant_id': 'object',
        'user_id': 'object',
        'name': 'str',
        'type': 'str',
        'sub_type': 'object',
        'sim_version': 'object',
        'creation_date': 'datetime',
        'modified_date': 'datetime',
        'properties': 'object',
        'data': 'object',
        'support_session': 'CanopyDocumentSupportSession',
        'notes': 'str',
        'delete_requested': 'bool',
        'parent_worksheet_id': 'object'
    }

    attribute_map = {
        'document_id': 'documentId',
        'tenant_id': 'tenantId',
        'user_id': 'userId',
        'name': 'name',
        'type': 'type',
        'sub_type': 'subType',
        'sim_version': 'simVersion',
        'creation_date': 'creationDate',
        'modified_date': 'modifiedDate',
        'properties': 'properties',
        'data': 'data',
        'support_session': 'supportSession',
        'notes': 'notes',
        'delete_requested': 'deleteRequested',
        'parent_worksheet_id': 'parentWorksheetId'
    }

    def __init__(self, document_id=None, tenant_id=None, user_id=None, name=None, type=None, sub_type=None, sim_version=None, creation_date=None, modified_date=None, properties=None, data=None, support_session=None, notes=None, delete_requested=None, parent_worksheet_id=None, local_vars_configuration=None):  # noqa: E501
        """GetConfigQueryResultConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._document_id = None
        self._tenant_id = None
        self._user_id = None
        self._name = None
        self._type = None
        self._sub_type = None
        self._sim_version = None
        self._creation_date = None
        self._modified_date = None
        self._properties = None
        self._data = None
        self._support_session = None
        self._notes = None
        self._delete_requested = None
        self._parent_worksheet_id = None
        self.discriminator = None

        self.document_id = document_id
        self.tenant_id = tenant_id
        self.user_id = user_id
        self.name = name
        self.type = type
        self.sub_type = sub_type
        self.sim_version = sim_version
        self.creation_date = creation_date
        self.modified_date = modified_date
        self.properties = properties
        self.data = data
        self.support_session = support_session
        self.notes = notes
        if delete_requested is not None:
            self.delete_requested = delete_requested
        self.parent_worksheet_id = parent_worksheet_id

    @property
    def document_id(self):
        """Gets the document_id of this GetConfigQueryResultConfig.  # noqa: E501


        :return: The document_id of this GetConfigQueryResultConfig.  # noqa: E501
        :rtype: object
        """
        return self._document_id

    @document_id.setter
    def document_id(self, document_id):
        """Sets the document_id of this GetConfigQueryResultConfig.


        :param document_id: The document_id of this GetConfigQueryResultConfig.  # noqa: E501
        :type document_id: object
        """
        if self.local_vars_configuration.client_side_validation and document_id is None:  # noqa: E501
            raise ValueError("Invalid value for `document_id`, must not be `None`")  # noqa: E501

        self._document_id = document_id

    @property
    def tenant_id(self):
        """Gets the tenant_id of this GetConfigQueryResultConfig.  # noqa: E501


        :return: The tenant_id of this GetConfigQueryResultConfig.  # noqa: E501
        :rtype: object
        """
        return self._tenant_id

    @tenant_id.setter
    def tenant_id(self, tenant_id):
        """Sets the tenant_id of this GetConfigQueryResultConfig.


        :param tenant_id: The tenant_id of this GetConfigQueryResultConfig.  # noqa: E501
        :type tenant_id: object
        """
        if self.local_vars_configuration.client_side_validation and tenant_id is None:  # noqa: E501
            raise ValueError("Invalid value for `tenant_id`, must not be `None`")  # noqa: E501

        self._tenant_id = tenant_id

    @property
    def user_id(self):
        """Gets the user_id of this GetConfigQueryResultConfig.  # noqa: E501


        :return: The user_id of this GetConfigQueryResultConfig.  # noqa: E501
        :rtype: object
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this GetConfigQueryResultConfig.


        :param user_id: The user_id of this GetConfigQueryResultConfig.  # noqa: E501
        :type user_id: object
        """
        if self.local_vars_configuration.client_side_validation and user_id is None:  # noqa: E501
            raise ValueError("Invalid value for `user_id`, must not be `None`")  # noqa: E501

        self._user_id = user_id

    @property
    def name(self):
        """Gets the name of this GetConfigQueryResultConfig.  # noqa: E501


        :return: The name of this GetConfigQueryResultConfig.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this GetConfigQueryResultConfig.


        :param name: The name of this GetConfigQueryResultConfig.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def type(self):
        """Gets the type of this GetConfigQueryResultConfig.  # noqa: E501


        :return: The type of this GetConfigQueryResultConfig.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this GetConfigQueryResultConfig.


        :param type: The type of this GetConfigQueryResultConfig.  # noqa: E501
        :type type: str
        """
        if self.local_vars_configuration.client_side_validation and type is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501

        self._type = type

    @property
    def sub_type(self):
        """Gets the sub_type of this GetConfigQueryResultConfig.  # noqa: E501


        :return: The sub_type of this GetConfigQueryResultConfig.  # noqa: E501
        :rtype: object
        """
        return self._sub_type

    @sub_type.setter
    def sub_type(self, sub_type):
        """Sets the sub_type of this GetConfigQueryResultConfig.


        :param sub_type: The sub_type of this GetConfigQueryResultConfig.  # noqa: E501
        :type sub_type: object
        """
        if self.local_vars_configuration.client_side_validation and sub_type is None:  # noqa: E501
            raise ValueError("Invalid value for `sub_type`, must not be `None`")  # noqa: E501

        self._sub_type = sub_type

    @property
    def sim_version(self):
        """Gets the sim_version of this GetConfigQueryResultConfig.  # noqa: E501


        :return: The sim_version of this GetConfigQueryResultConfig.  # noqa: E501
        :rtype: object
        """
        return self._sim_version

    @sim_version.setter
    def sim_version(self, sim_version):
        """Sets the sim_version of this GetConfigQueryResultConfig.


        :param sim_version: The sim_version of this GetConfigQueryResultConfig.  # noqa: E501
        :type sim_version: object
        """
        if self.local_vars_configuration.client_side_validation and sim_version is None:  # noqa: E501
            raise ValueError("Invalid value for `sim_version`, must not be `None`")  # noqa: E501

        self._sim_version = sim_version

    @property
    def creation_date(self):
        """Gets the creation_date of this GetConfigQueryResultConfig.  # noqa: E501


        :return: The creation_date of this GetConfigQueryResultConfig.  # noqa: E501
        :rtype: datetime
        """
        return self._creation_date

    @creation_date.setter
    def creation_date(self, creation_date):
        """Sets the creation_date of this GetConfigQueryResultConfig.


        :param creation_date: The creation_date of this GetConfigQueryResultConfig.  # noqa: E501
        :type creation_date: datetime
        """
        if self.local_vars_configuration.client_side_validation and creation_date is None:  # noqa: E501
            raise ValueError("Invalid value for `creation_date`, must not be `None`")  # noqa: E501

        self._creation_date = creation_date

    @property
    def modified_date(self):
        """Gets the modified_date of this GetConfigQueryResultConfig.  # noqa: E501


        :return: The modified_date of this GetConfigQueryResultConfig.  # noqa: E501
        :rtype: datetime
        """
        return self._modified_date

    @modified_date.setter
    def modified_date(self, modified_date):
        """Sets the modified_date of this GetConfigQueryResultConfig.


        :param modified_date: The modified_date of this GetConfigQueryResultConfig.  # noqa: E501
        :type modified_date: datetime
        """
        if self.local_vars_configuration.client_side_validation and modified_date is None:  # noqa: E501
            raise ValueError("Invalid value for `modified_date`, must not be `None`")  # noqa: E501

        self._modified_date = modified_date

    @property
    def properties(self):
        """Gets the properties of this GetConfigQueryResultConfig.  # noqa: E501


        :return: The properties of this GetConfigQueryResultConfig.  # noqa: E501
        :rtype: object
        """
        return self._properties

    @properties.setter
    def properties(self, properties):
        """Sets the properties of this GetConfigQueryResultConfig.


        :param properties: The properties of this GetConfigQueryResultConfig.  # noqa: E501
        :type properties: object
        """

        self._properties = properties

    @property
    def data(self):
        """Gets the data of this GetConfigQueryResultConfig.  # noqa: E501


        :return: The data of this GetConfigQueryResultConfig.  # noqa: E501
        :rtype: object
        """
        return self._data

    @data.setter
    def data(self, data):
        """Sets the data of this GetConfigQueryResultConfig.


        :param data: The data of this GetConfigQueryResultConfig.  # noqa: E501
        :type data: object
        """

        self._data = data

    @property
    def support_session(self):
        """Gets the support_session of this GetConfigQueryResultConfig.  # noqa: E501


        :return: The support_session of this GetConfigQueryResultConfig.  # noqa: E501
        :rtype: CanopyDocumentSupportSession
        """
        return self._support_session

    @support_session.setter
    def support_session(self, support_session):
        """Sets the support_session of this GetConfigQueryResultConfig.


        :param support_session: The support_session of this GetConfigQueryResultConfig.  # noqa: E501
        :type support_session: CanopyDocumentSupportSession
        """

        self._support_session = support_session

    @property
    def notes(self):
        """Gets the notes of this GetConfigQueryResultConfig.  # noqa: E501


        :return: The notes of this GetConfigQueryResultConfig.  # noqa: E501
        :rtype: str
        """
        return self._notes

    @notes.setter
    def notes(self, notes):
        """Sets the notes of this GetConfigQueryResultConfig.


        :param notes: The notes of this GetConfigQueryResultConfig.  # noqa: E501
        :type notes: str
        """

        self._notes = notes

    @property
    def delete_requested(self):
        """Gets the delete_requested of this GetConfigQueryResultConfig.  # noqa: E501


        :return: The delete_requested of this GetConfigQueryResultConfig.  # noqa: E501
        :rtype: bool
        """
        return self._delete_requested

    @delete_requested.setter
    def delete_requested(self, delete_requested):
        """Sets the delete_requested of this GetConfigQueryResultConfig.


        :param delete_requested: The delete_requested of this GetConfigQueryResultConfig.  # noqa: E501
        :type delete_requested: bool
        """

        self._delete_requested = delete_requested

    @property
    def parent_worksheet_id(self):
        """Gets the parent_worksheet_id of this GetConfigQueryResultConfig.  # noqa: E501


        :return: The parent_worksheet_id of this GetConfigQueryResultConfig.  # noqa: E501
        :rtype: object
        """
        return self._parent_worksheet_id

    @parent_worksheet_id.setter
    def parent_worksheet_id(self, parent_worksheet_id):
        """Sets the parent_worksheet_id of this GetConfigQueryResultConfig.


        :param parent_worksheet_id: The parent_worksheet_id of this GetConfigQueryResultConfig.  # noqa: E501
        :type parent_worksheet_id: object
        """

        self._parent_worksheet_id = parent_worksheet_id

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GetConfigQueryResultConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, GetConfigQueryResultConfig):
            return True

        return self.to_dict() != other.to_dict()



import canopy


def get_study_document(session: canopy.Session, canopy_document: canopy.openapi.CanopyDocument) -> canopy.openapi.StudyDocument:
    # We access the protected member here to save us modifying the swagger generated code.
    return session.sync_client._ApiClient__deserialize(canopy_document.data, 'StudyDocument')



import unittest
import canopy


class TenantUsersTest(unittest.TestCase):

    def setUp(self):
        self.users = [
            canopy.openapi.GetTenantUsersQueryResultUserItem('id0', 'u0', 'u0@d.com', roles=[], is_enabled=True),
            canopy.openapi.GetTenantUsersQueryResultUserItem('id1', 'u1', 'u1@d.com', roles=[], is_enabled=True),
            canopy.openapi.GetTenantUsersQueryResultUserItem('id2', 'u2', 'u2@d.com', roles=[], is_enabled=True),
        ]

        self.target = canopy.TenantUsers(
            canopy.openapi.GetTenantUsersQueryResult(self.users))

    def test_it_should_contain_list_of_users(self):
        self.assertSequenceEqual(self.users, self.target.list)

    def test_it_should_return_by_user_id(self):
        self.assertEqual(
            self.users[1],
            self.target.get_by_user_id('id1'))

    def test_it_should_return_by_username(self):
        self.assertEqual(
            self.users[1],
            self.target.get_by_username('u1'))

    def test_it_should_throw_if_user_id_not_found(self):
        with self.assertRaises(canopy.NotFoundError):
            self.target.get_by_user_id('blah')

    def test_it_should_throw_if_username_not_found(self):
        with self.assertRaises(canopy.NotFoundError):
            self.target.get_by_username('blah')



from typing import Optional, Sequence

import pandas as pd
import logging

logger = logging.getLogger(__name__)

units_column_names = ['units']


def process_data_frame(
        data_frame: Optional[pd.DataFrame],
        index_column_name: Optional[str] = None,
        empty_string_valid_column_names: Optional[Sequence[str]] = None) -> Optional[pd.DataFrame]:

    if data_frame is None:
        return

    if index_column_name is not None:
        data_frame.set_index([index_column_name], inplace=True)

    columns = data_frame.columns

    empty_string_valid_column_names_set = set(units_column_names)
    if empty_string_valid_column_names is not None:
        empty_string_valid_column_names_set.update(empty_string_valid_column_names)

    # When using pandas.read_csv it will set empty strings to NaN, so here we
    # force them to empty strings if requested.
    for column_name in empty_string_valid_column_names_set:
        if column_name in columns:
            data_frame[column_name] = data_frame[column_name].fillna('')

    for column_name in units_column_names:
        if column_name in columns:
            data_frame[column_name] = data_frame[column_name].replace('()', '')

    return data_frame



import unittest
import canopy


class ProxyConfigurationTest(unittest.TestCase):

    def test_it_should_throw_if_scheme_not_found(self):
        with self.assertRaises(RuntimeError):
            canopy.ProxyConfiguration('test.com')

    def test_it_should_create_auth_url_and_headers_when_no_auth(self):
        proxy = canopy.ProxyConfiguration('https://test.com')
        self.assertEqual(proxy.url, 'https://test.com')
        self.assertEqual(proxy.auth_url, 'https://test.com')
        self.assertIsNone(proxy.username)
        self.assertIsNone(proxy.password)
        self.assertIsNone(proxy.headers)

    def test_it_should_create_auth_url_and_headers_when_auth(self):
        proxy = canopy.ProxyConfiguration('https://test.com', 'u1', 'p1')
        self.assertEqual(proxy.url, 'https://test.com')
        self.assertEqual(proxy.auth_url, 'https://u1:p1@test.com')
        self.assertEqual(proxy.username, 'u1')
        self.assertEqual(proxy.password, 'p1')
        self.assertIsNotNone(proxy.headers)




# coding: utf-8

"""
    Canopy.Api

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from canopy.openapi.configuration import Configuration


class GetAdminTenantSettingsQueryResultSettings(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'study_types': 'list[str]',
        'tags': 'list[str]',
        'internal_tags': 'list[str]',
        'sim_version': 'object',
        'pool_settings': 'AdminTenantSettingsPoolSettings',
        'secondary_pool_settings': 'AdminTenantSettingsPoolSettings',
        'heavy_pool_settings': 'AdminTenantSettingsPoolSettings'
    }

    attribute_map = {
        'study_types': 'studyTypes',
        'tags': 'tags',
        'internal_tags': 'internalTags',
        'sim_version': 'simVersion',
        'pool_settings': 'poolSettings',
        'secondary_pool_settings': 'secondaryPoolSettings',
        'heavy_pool_settings': 'heavyPoolSettings'
    }

    def __init__(self, study_types=None, tags=None, internal_tags=None, sim_version=None, pool_settings=None, secondary_pool_settings=None, heavy_pool_settings=None, local_vars_configuration=None):  # noqa: E501
        """GetAdminTenantSettingsQueryResultSettings - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._study_types = None
        self._tags = None
        self._internal_tags = None
        self._sim_version = None
        self._pool_settings = None
        self._secondary_pool_settings = None
        self._heavy_pool_settings = None
        self.discriminator = None

        self.study_types = study_types
        self.tags = tags
        self.internal_tags = internal_tags
        self.sim_version = sim_version
        self.pool_settings = pool_settings
        self.secondary_pool_settings = secondary_pool_settings
        self.heavy_pool_settings = heavy_pool_settings

    @property
    def study_types(self):
        """Gets the study_types of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501


        :return: The study_types of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501
        :rtype: list[str]
        """
        return self._study_types

    @study_types.setter
    def study_types(self, study_types):
        """Sets the study_types of this GetAdminTenantSettingsQueryResultSettings.


        :param study_types: The study_types of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501
        :type study_types: list[str]
        """

        self._study_types = study_types

    @property
    def tags(self):
        """Gets the tags of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501


        :return: The tags of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501
        :rtype: list[str]
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """Sets the tags of this GetAdminTenantSettingsQueryResultSettings.


        :param tags: The tags of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501
        :type tags: list[str]
        """

        self._tags = tags

    @property
    def internal_tags(self):
        """Gets the internal_tags of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501


        :return: The internal_tags of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501
        :rtype: list[str]
        """
        return self._internal_tags

    @internal_tags.setter
    def internal_tags(self, internal_tags):
        """Sets the internal_tags of this GetAdminTenantSettingsQueryResultSettings.


        :param internal_tags: The internal_tags of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501
        :type internal_tags: list[str]
        """

        self._internal_tags = internal_tags

    @property
    def sim_version(self):
        """Gets the sim_version of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501


        :return: The sim_version of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501
        :rtype: object
        """
        return self._sim_version

    @sim_version.setter
    def sim_version(self, sim_version):
        """Sets the sim_version of this GetAdminTenantSettingsQueryResultSettings.


        :param sim_version: The sim_version of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501
        :type sim_version: object
        """

        self._sim_version = sim_version

    @property
    def pool_settings(self):
        """Gets the pool_settings of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501


        :return: The pool_settings of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501
        :rtype: AdminTenantSettingsPoolSettings
        """
        return self._pool_settings

    @pool_settings.setter
    def pool_settings(self, pool_settings):
        """Sets the pool_settings of this GetAdminTenantSettingsQueryResultSettings.


        :param pool_settings: The pool_settings of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501
        :type pool_settings: AdminTenantSettingsPoolSettings
        """

        self._pool_settings = pool_settings

    @property
    def secondary_pool_settings(self):
        """Gets the secondary_pool_settings of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501


        :return: The secondary_pool_settings of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501
        :rtype: AdminTenantSettingsPoolSettings
        """
        return self._secondary_pool_settings

    @secondary_pool_settings.setter
    def secondary_pool_settings(self, secondary_pool_settings):
        """Sets the secondary_pool_settings of this GetAdminTenantSettingsQueryResultSettings.


        :param secondary_pool_settings: The secondary_pool_settings of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501
        :type secondary_pool_settings: AdminTenantSettingsPoolSettings
        """

        self._secondary_pool_settings = secondary_pool_settings

    @property
    def heavy_pool_settings(self):
        """Gets the heavy_pool_settings of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501


        :return: The heavy_pool_settings of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501
        :rtype: AdminTenantSettingsPoolSettings
        """
        return self._heavy_pool_settings

    @heavy_pool_settings.setter
    def heavy_pool_settings(self, heavy_pool_settings):
        """Sets the heavy_pool_settings of this GetAdminTenantSettingsQueryResultSettings.


        :param heavy_pool_settings: The heavy_pool_settings of this GetAdminTenantSettingsQueryResultSettings.  # noqa: E501
        :type heavy_pool_settings: AdminTenantSettingsPoolSettings
        """

        self._heavy_pool_settings = heavy_pool_settings

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GetAdminTenantSettingsQueryResultSettings):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, GetAdminTenantSettingsQueryResultSettings):
            return True

        return self.to_dict() != other.to_dict()



import canopy


def get_default_config_path(
        session: canopy.Session,
        config_type: str,
        document_name: str):

    config_type_metadata = session.study_types.get_config_type_metadata(config_type)
    return '{}/{}.json'.format(config_type_metadata.plural_key, document_name)




import canopy.openapi
import canopy.openapi_asyncio

from canopy.not_found_error import NotFoundError
from canopy.request_with_retry import request_with_retry

from canopy.constants import Constants
from canopy.defined_kwargs import defined_kwargs
from canopy.proxy_configuration import ProxyConfiguration
from canopy.authentication_data import AuthenticationData
from canopy.authentication import Authentication
from canopy.user_settings_cache import UserSettingsCache
from canopy.units import Units
from canopy.tenant_users import TenantUsers
from canopy.tenant_users_cache import TenantUsersCache
from canopy.tenant_sim_version_cache import TenantSimVersionCache
from canopy.study_types_cache import StudyTypesCache
from canopy.study_scalar_results import StudyScalarResults
from canopy.session import Session

from canopy.dynamic_dict_to_object import DynamicDictToObject
from canopy.study_job_result import StudyJobResult
from canopy.study_result import StudyResult
from canopy.config_result import ConfigResult
from canopy.loaded_channel import LoadedChannel
from canopy.local_config import LocalConfig
from canopy.prompt_for_authentication import prompt_for_authentication
from canopy.process_data_frame import process_data_frame
from canopy.try_load_csv_from_url import try_load_csv_from_url

from canopy.run import run
from canopy.serializable_value import SerializableValue
from canopy.dict_to_object import dict_to_object
from canopy.sim_version_to_number import sim_version_to_number
from canopy.ensure_dict import ensure_dict
from canopy.ensure_sim_type_string import ensure_sim_type_string
from canopy.ensure_study_type_string import ensure_study_type_string
from canopy.properties_dict_to_list import properties_dict_to_list
from canopy.get_study_type_definition_for_sim_version import get_study_type_definition_for_sim_version
from canopy.create_list_filter import create_list_filter
from canopy.load_config import load_config
from canopy.update_config import update_config
from canopy.load_study import load_study
from canopy.load_study_job import load_study_job
from canopy.load_channel import load_channel
from canopy.load_vector_metadata import load_vector_metadata
from canopy.get_study_document import get_study_document
from canopy.job_count_to_simulation_count import job_count_to_simulation_count
from canopy.load_study_job_scalar_results import load_study_job_scalar_results
from canopy.load_study_scalar_results import load_study_scalar_results
from canopy.find_config import find_config
from canopy.find_study import find_study
from canopy.create_config import create_config
from canopy.create_study import create_study
from canopy.wait_for_study import wait_for_study
from canopy.get_default_config_path import get_default_config_path
from canopy.load_default_config import load_default_config
from canopy.delete_config import delete_config
from canopy.delete_study import delete_study



import math
import pandas as pd
import numpy as np
from typing import Optional, Union


class UnitDefinition:
    def __init__(self, target_unit: str, si_unit: str, factor: float = 1, offset: float = 0):
        self.target_unit = target_unit
        self.si_unit = si_unit
        self.factor = factor
        self.offset = offset


class ConversionBetweenUnits:
    def __init__(
          self,
          factor_to_si: float,
          offset_to_si: float,
          factor_from_si: float,
          offset_from_si: float,
          is_conversion_required: bool):
        self.factor_to_si = factor_to_si
        self.offset_to_si = offset_to_si
        self.factor_from_si = factor_from_si
        self.offset_from_si = offset_from_si
        self.is_conversion_required = is_conversion_required


def create_default_units():
    units_list = [
        # list SI ones first so they are always at the top of the suggested list
        UnitDefinition('()', '()', factor=1, offset=0),
        UnitDefinition('m', 'm', factor=1, offset=0),
        UnitDefinition('kg', 'kg', factor=1, offset=0),
        UnitDefinition('N', 'N', factor=1, offset=0),
        UnitDefinition('m/s', 'm/s', factor=1, offset=0),
        UnitDefinition('rad', 'rad', factor=1, offset=0),
        UnitDefinition('Pa', 'Pa', factor=1, offset=0),
        UnitDefinition('rad/s', 'rad/s', factor=1, offset=0),
        UnitDefinition('Nm', 'Nm', factor=1, offset=0),
        UnitDefinition('m/s2', 'm/s2', factor=1, offset=0),
        UnitDefinition('W', 'W', factor=1, offset=0),
        UnitDefinition('J', 'J', factor=1, offset=0),
        UnitDefinition('s', 's', factor=1, offset=0),
        UnitDefinition('K', 'K', factor=1, offset=0),
        UnitDefinition('N/rad', 'N/rad', factor=1, offset=0),
        UnitDefinition('Nm/rad', 'Nm/rad', factor=1, offset=0),
        UnitDefinition('N/m', 'N/m', factor=1, offset=0),
        UnitDefinition('rad/s2', 'rad/s2', factor=1, offset=0),
        UnitDefinition('kg/s', 'kg/s', factor=1, offset=0),
        UnitDefinition('Ns/m', 'Ns/m', factor=1, offset=0),

        # now start listing all the horrible non-SI ones that people like
        UnitDefinition('', '()', factor=1, offset=0),
        UnitDefinition('%', '()', factor=0.01, offset=0),
        UnitDefinition('pt', '()', factor=0.01, offset=0),
        UnitDefinition('e-3', '()', factor=1e-3, offset=0),
        UnitDefinition('e-6', '()', factor=1e-6, offset=0),

        UnitDefinition('N/m2', 'Pa', factor=1, offset=0),
        UnitDefinition('bar', 'Pa', factor=1e5, offset=0),
        UnitDefinition('mbar', 'Pa', factor=1e5 * 0.001, offset=0),
        UnitDefinition('psi', 'Pa', factor=6894.76, offset=0),
        UnitDefinition('inHg', 'Pa', factor=3386.39, offset=0),

        UnitDefinition('gm', 'kg', factor=0.001, offset=0),
        UnitDefinition('lbm', 'kg', factor=0.45359237, offset=0),
        UnitDefinition('st', 'kg', factor=6.35029318, offset=0),

        UnitDefinition('lbf', 'N', factor=4.448222, offset=0),
        UnitDefinition('kN', 'N', factor=1e3, offset=0),
        UnitDefinition('MN', 'N', factor=1e6, offset=0),
        UnitDefinition('GN', 'N', factor=1e9, offset=0),

        UnitDefinition('ms', 's', factor=0.001, offset=0),
        UnitDefinition('ks', 's', factor=1000, offset=0),

        UnitDefinition('mm', 'm', factor=0.001, offset=0),
        UnitDefinition('cm', 'm', factor=0.01, offset=0),
        UnitDefinition('km', 'm', factor=1000, offset=0),
        UnitDefinition('miles', 'm', factor=1609.344, offset=0),
        UnitDefinition('in', 'm', factor=0.0254, offset=0),
        UnitDefinition('ft', 'm', factor=0.3048, offset=0),
        UnitDefinition('yd', 'm', factor=0.9144, offset=0),

        UnitDefinition('kph', 'm/s', factor=1 / 3.6, offset=0),
        UnitDefinition('mph', 'm/s', factor=0.44704, offset=0),
        UnitDefinition('mm/s', 'm/s', factor=0.001, offset=0),
        UnitDefinition('kn', 'm/s', factor=0.514444, offset=0),
        UnitDefinition('in/s', 'm/s', factor=0.0254, offset=0),

        UnitDefinition('g', 'm/s2', factor=9.80665, offset=0),

        UnitDefinition('deg', 'rad', factor=math.pi / 180, offset=0),
        UnitDefinition('°', 'rad', factor=math.pi / 180, offset=0),

        UnitDefinition('rpm', 'rad/s', factor=math.pi / 30, offset=0),
        UnitDefinition('deg/s', 'rad/s', factor=math.pi / 180, offset=0),
        UnitDefinition('°/s', 'rad/s', factor=math.pi / 180, offset=0),

        UnitDefinition('Nm/deg', 'Nm/rad', factor=180 / math.pi, offset=0),

        UnitDefinition('kW', 'W', factor=1e3, offset=0),
        UnitDefinition('MW', 'W', factor=1e6, offset=0),
        UnitDefinition('GW', 'W', factor=1e9, offset=0),

        UnitDefinition('kJ', 'J', factor=1e3, offset=0),
        UnitDefinition('MJ', 'J', factor=1e6, offset=0),
        UnitDefinition('GJ', 'J', factor=1e9, offset=0),
        UnitDefinition('kWh', 'J', factor=1e3 * 60 * 60, offset=0),

        UnitDefinition('kPa', 'Pa', factor=1e3, offset=0),
        UnitDefinition('MPa', 'Pa', factor=1e6, offset=0),
        UnitDefinition('GPa', 'Pa', factor=1e9, offset=0),

        UnitDefinition('kNm', 'Nm', factor=1e3, offset=0),
        UnitDefinition('MNm', 'Nm', factor=1e6, offset=0),
        UnitDefinition('GNm', 'Nm', factor=1e9, offset=0),

        UnitDefinition('C', 'K', factor=1, offset=273.15),
        UnitDefinition('°C', 'K', factor=1, offset=273.15),
        UnitDefinition('F', 'K', factor=5 / 9, offset=459.67 * 5 / 9),

        UnitDefinition('N/mm', 'N/m', factor=1000, offset=0),
        UnitDefinition('lbf/in', 'N/m', factor=175.127, offset=0),

        UnitDefinition('lbfs/in', 'Ns/m', factor=175.127, offset=0),
    ]

    return {units_list[i].target_unit: units_list[i] for i in range(0, len(units_list))}


class Units(object):
    def __init__(self):
        self.to_si_table = create_default_units()

    def try_get_conversion_to_si(self, unit: str) -> Optional[UnitDefinition]:
        if unit in self.to_si_table:
            return self.to_si_table[unit]

        return None

    def get_conversion_to_si(self, unit: str) -> UnitDefinition:
        return self.to_si_table[unit]

    def get_conversion_to_si_or_default(self, unit: str) -> UnitDefinition:
        result = self.try_get_conversion_to_si(unit)
        if result is None:
            return UnitDefinition(unit, unit, 1, 0)

        return result

    def convert_value_to_si(self, value: float, source_unit: str, apply_factor_only: bool = False) -> float:
        conversion_to_si = self.get_conversion_to_si(source_unit)
        factor = conversion_to_si.factor
        offset = 0 if apply_factor_only else conversion_to_si.offset
        return value * factor + offset

    def convert_value_from_si(self, value: float, target_unit: str, apply_factor_only: bool = False) -> float:
        conversion_to_si = self.get_conversion_to_si(target_unit)
        factor = conversion_to_si.factor
        offset = 0 if apply_factor_only else conversion_to_si.offset
        return (value - offset) / factor

    def convert_value_between_units(self, value: float, source_unit: str, target_unit: str, apply_factor_only: bool = False) -> float:
        return self.convert_value_from_si(self.convert_value_to_si(value, source_unit, apply_factor_only), target_unit, apply_factor_only)

    def _convert_values_to_si(self, values: Union[np.array, pd.Series], source_unit: str, inplace: bool = False) -> Union[np.array, pd.Series]:
        if values is None:
            return None

        conversion_to_si = self.get_conversion_to_si(source_unit)
        factor = conversion_to_si.factor
        offset = conversion_to_si.offset

        result = values

        if inplace:
            if factor != 1:
                result *= factor

            if offset != 0:
                result += offset
        else:
            if factor != 1:
                result = result * factor

            if offset != 0:
                result = result + offset

        return result

    def convert_array_to_si(self, values: np.array, source_unit: str, inplace: bool = False, always_return_copy: bool = False) -> np.array:
        result = self._convert_values_to_si(values, source_unit, inplace=inplace)
        return np.copy(result) if result is not None and result is values and always_return_copy else result

    def convert_series_to_si(self, values: pd.Series, source_unit: str, inplace: bool = False, always_return_copy: bool = False) -> pd.Series:
        result = self._convert_values_to_si(values, source_unit, inplace=inplace)
        return result.copy() if result is not None and result is values and always_return_copy else result

    def _convert_values_from_si(self, values: Union[np.array, pd.Series], target_unit: str, inplace: bool = False) -> Union[np.array, pd.Series]:
        if values is None:
            return None

        conversion_to_si = self.get_conversion_to_si(target_unit)
        factor = conversion_to_si.factor
        offset = conversion_to_si.offset

        result = values

        if inplace:
            if offset != 0:
                result -= offset

            if factor != 1:
                result /= factor
        else:
            if offset != 0:
                result = result - offset

            if factor != 1:
                result = result / factor

        return result

    def convert_array_from_si(self, values: np.array, target_unit: str, inplace: bool = False, always_return_copy: bool = False) -> np.array:
        result = self._convert_values_from_si(values, target_unit, inplace=inplace)
        return np.copy(result) if result is not None and result is values and always_return_copy else result

    def convert_series_from_si(self, values: pd.Series, target_unit: str, inplace: bool = False, always_return_copy: bool = False) -> pd.Series:
        result = self._convert_values_from_si(values, target_unit, inplace=inplace)
        return result.copy() if result is not None and result is values and always_return_copy else result

    def _convert_values_between_units(self, values: Union[np.array, pd.Series], source_unit: str, target_unit: str, inplace: bool = False) -> Union[np.array, pd.Series]:
        if values is None:
            return None

        conversion = self.get_conversion_between_units(source_unit, target_unit)

        result = values

        if not conversion.is_conversion_required:
            return result

        combined_offset = conversion.offset_to_si - conversion.offset_from_si

        if inplace:
            if conversion.factor_to_si != 1:
                result *= conversion.factor_to_si

            if combined_offset != 0:
                result += combined_offset

            if conversion.factor_from_si != 1:
                result /= conversion.factor_from_si
        else:
            if conversion.factor_to_si != 1:
                result = result * conversion.factor_to_si

            if combined_offset != 0:
                result = result + combined_offset

            if conversion.factor_from_si != 1:
                result = result / conversion.factor_from_si

        return result

    def convert_array_between_units(self, values: np.array, source_unit: str, target_unit: str, inplace: bool = False, always_return_copy: bool = False) -> np.array:
        result = self._convert_values_between_units(values, source_unit, target_unit, inplace=inplace)
        return np.copy(result) if result is not None and result is values and always_return_copy else result

    def convert_series_between_units(self, values: pd.Series, source_unit: str, target_unit: str, inplace: bool = False, always_return_copy: bool = False) -> pd.Series:
        result = self._convert_values_between_units(values, source_unit, target_unit, inplace=inplace)
        return result.copy() if result is not None and result is values and always_return_copy else result

    def get_conversion_between_units(self, source_unit: str, target_unit: str) -> ConversionBetweenUnits:
        conversion_from_source_to_si = self.get_conversion_to_si(source_unit)
        factor_to_si = conversion_from_source_to_si.factor
        offset_to_si = conversion_from_source_to_si.offset

        conversion_from_target_to_si = self.get_conversion_to_si(target_unit)
        factor_from_si = conversion_from_target_to_si.factor
        offset_from_si = conversion_from_target_to_si.offset

        conversion_required = True

        combined_offset = offset_to_si - offset_from_si
        if combined_offset == 0:
            combined_factor = factor_to_si / factor_from_si

            if combined_factor == 1:
                conversion_required = False

        return ConversionBetweenUnits(factor_to_si, offset_to_si, factor_from_si, offset_from_si, conversion_required)
